CONN sec_mgr/oracle10g
CREATE USER user1 IDENTIFIED BY NO;
CONN scott/tiger
-- rename existing table
ALTER TABLE emp RENAME TO emp$;
-- create view that removes sensitive columns
CREATE VIEW emp
AS
  SELECT ename, job
    FROM emp$;
-- grant access to view
GRANT SELECT ON emp TO user1;
-- revoke access from table
REVOKE SELECT ON emp$ FROM user1;

-- undo
DROP VIEW emp;
ALTER TABLE emp$ RENAME TO emp;


-------------- Column masking ----------
DROP TABLE people;
CREATE TABLE people
AS SELECT ename username, job, sal salary, deptno
  FROM emp
  WHERE deptno = 20;
-- create view that only returns the salary
-- for the user issuing the query
CREATE OR REPLACE VIEW people_cls
AS
  SELECT username,
         job,
         deptno,
         DECODE (UPPER (username),
                 USER, salary,
                 NULL) salary
    FROM people;

COL username format a15
SELECT * FROM people_cls;

ALTER TABLE people ADD (phone_number VARCHAR2(20));
GRANT UPDATE (phone_number) ON people TO user1;

UPDATE scott.people
   SET phone_number = '555-1212';

---------------- instead of trigger ------------
drop trigger people_sal_update;
-- Computing salaries for everyone will only
-- return user's salary. This is good security.
SELECT SUM (salary) FROM people_cls;
-- Give everyone a 10% raise.
-- This will fail on the derived column
UPDATE people_cls
   SET salary = salary * 1.1;

-- Create "instead of" trigger to perform updates on 
-- base table. Trigger code is based on view security.
-- If user can see the salary, then update the record.
CREATE OR REPLACE TRIGGER people_sal_update
  INSTEAD OF UPDATE
  ON people_cls
  FOR EACH ROW
DECLARE
BEGIN
  -- If salary is not null, then user has read access
  -- so perform update to base table
  IF :OLD.salary IS NOT NULL
  THEN
    -- Update base table
    UPDATE people
       SET salary = :NEW.salary
     WHERE username = :NEW.username;
  END IF;
END;
/
-- Display current values.
SELECT username, salary FROM people;
-- Give everyone a 10% raise by updating view.
-- This will succeed now the trigger has been created.
-- Only records updated will be invoking users.
UPDATE people_cls
   SET salary = salary * 1.1;
-- show affects of update
SELECT username, salary FROM people;

SELECT SUM (salary) FROM people_cls;
ROLLBACK ;
