spool cls_views.log

-- create context to store our view source
@conn demo/demo

CREATE TABLE EMP_TEST ( 
  EMPNO     NUMBER (4)    NOT NULL, 
  ENAME     VARCHAR2 (10), 
  JOB       VARCHAR2 (9), 
  MGR       NUMBER (4), 
  HIREDATE  DATE, 
  SAL       NUMBER (7,2), 
  COMM      NUMBER (7,2), 
  DEPTNO    NUMBER (2), 
  CONSTRAINT PK_EMP
  PRIMARY KEY ( EMPNO ) ) ; 

-- create view:  decode to show sal if it is user's record, else show NULL.
create or replace view emp
as select empno, ename, job, hiredate, deptno, mgr, decode(upper(ename),USER,to_char(sal),NULL) sal from EMP_TEST;

-- grant column level access control to allow user to update
-- ename column.  This allows all rows to be updated
grant select, update (ename) on emp to scott;

-- create VPD policy to eliminate all rows except users
create or replace function pred_function
( p_schema in varchar2, p_object in varchar2 )
return varchar2
as
begin
  return 'upper(ename) = USER';
end;
/

-- apply VPD to updates
@conn system/manager
begin 
  DBMS_RLS.ADD_POLICY ( OBJECT_SCHEMA   => 'DEMO',
                        OBJECT_NAME     => 'EMP',
                        POLICY_NAME     => 'CLS_TEST',
                        FUNCTION_SCHEMA => 'DEMO',
                        POLICY_FUNCTION => 'PRED_FUNCTION',
                        STATEMENT_TYPES => 'UPDATE');
end;
/

@conn demo/demo
set feedback off
set echo off
prompt inserting records ...

INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7369, 'DEMO', 'CLERK', 7902,  TO_Date( '12/17/1980 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 800, NULL, 20); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7499, 'ALLEN', 'SALESMAN', 7698,  TO_Date( '02/20/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1600, 300, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7521, 'WARD', 'SALESMAN', 7698,  TO_Date( '02/22/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1250, 500, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7566, 'JONES', 'MANAGER', 7839,  TO_Date( '04/02/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 2975, NULL, 20); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7654, 'MARTIN', 'SALESMAN', 7698,  TO_Date( '09/28/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1250, 1400, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7698, 'BLAKE', 'MANAGER', 7839,  TO_Date( '05/01/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 2850, NULL, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7782, 'CLARK', 'MANAGER', 7839,  TO_Date( '06/09/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 2450, NULL, 10); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7788, 'SCOTT', 'ANALYST', 7566,  TO_Date( '04/19/1987 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 3000, NULL, 20); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7839, 'KING', 'PRESIDENT', NULL,  TO_Date( '11/17/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 5000, NULL, 10); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7844, 'TURNER', 'SALESMAN', 7698,  TO_Date( '09/08/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1500, 0, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7876, 'ADAMS', 'CLERK', 7788,  TO_Date( '05/23/1987 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1100, NULL, 20); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7900, 'JAMES', 'CLERK', 7698,  TO_Date( '12/03/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 950, NULL, 30); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7902, 'FORD', 'ANALYST', 7566,  TO_Date( '12/03/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 3000, NULL, 20); 
INSERT INTO EMP_TEST ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM,
DEPTNO ) VALUES ( 
7934, 'MILLER', 'CLERK', 7782,  TO_Date( '01/23/1982 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM')
, 1300, NULL, 10); 
COMMIT;

set feedback on
set echo on

-- show user demo
select ename, sal from emp;

-- show scott
@conn scott/tiger
select ename, sal from demo.emp;

update demo.emp set ename = lower(ename);
select  ename, sal from demo.emp;

-- show other updates don't work
update  demo.emp set sal = 5000;

spool off;
@conn demo/demo
drop function pred_function;
drop table emp_test;
drop view emp;
edit cls_views.log
exit
