/*
conn sys/oracle as sysdba
grant select on DBA_POLICIES to sec_mgr;
grant execute any procedure to sec_mgr;
grant select any table to sec_mgr;
conn sec_mgr/sec_mgr
*/

-- Need the following privileges
--   SELECT on DBA_POLICIES
--   EXECUTE ANY PROCEDURE
--   SELECT ANY TABLE
CREATE OR REPLACE PROCEDURE debug_policy (
  p_schema  IN  VARCHAR2,
  p_object  IN  VARCHAR2,
  p_policy  IN  VARCHAR2)
AS
/*
 * Not handling long predicates
 */
  l_sql_str    VARCHAR2 (32767);
  l_package    VARCHAR2 (32);
  l_predicate  VARCHAR2 (4000)  := '';
  l_errm       VARCHAR2 (4000);
BEGIN
  FOR rec IN (SELECT *
                FROM dba_policies
               WHERE long_predicate = 'NO'
                 AND object_owner =
                                  UPPER (p_schema)
                 AND object_name =
                                  UPPER (p_object)
                 AND policy_name =
                                  UPPER (p_policy))
  LOOP
    IF (rec.PACKAGE IS NOT NULL)
    THEN
      l_package := rec.PACKAGE || '.';
    END IF;

    EXECUTE IMMEDIATE    'select '
                      || rec.pf_owner
                      || '.'
                      || l_package
                      || rec.FUNCTION
                      || '('''
                      || p_schema
                      || ''','''
                      || p_object
                      || ''')'
                      || ' from dual'
                 INTO l_predicate;
    l_sql_str :=
         'select * from '
      || p_schema
      || '.'
      || p_object
      || ' where '
      || l_predicate;

    BEGIN
      EXECUTE IMMEDIATE l_sql_str;
      DBMS_OUTPUT.put_line
             ('- The SQL executed successfully -');
      DBMS_OUTPUT.put_line (l_sql_str);
    EXCEPTION
      WHEN OTHERS
      THEN
        DBMS_OUTPUT.put_line (CHR (10));
        DBMS_OUTPUT.put_line
                 ('- Errors were found for SQL -');
        DBMS_OUTPUT.put_line (CHR (10));
        DBMS_OUTPUT.put_line (l_sql_str);
        l_errm := SQLERRM;
        DBMS_OUTPUT.put_line (l_errm);
    END;
  END LOOP;
END;
/

SHO errors

EXEC debug_policy('SCOTT','EMP','debug')
