SPOOL seq.log
CONN sec_mgr/oracle10g
CREATE TABLE people AS SELECT * FROM scott.emp;
CREATE SYNONYM people_syn FOR people;
CREATE VIEW people_view
AS
  SELECT *
    FROM people_syn;

CREATE TABLE access_tab (
username    VARCHAR2(30),
access_date DATE,
sql_text    CLOB,
audit_id    NUMBER,
trace_route CLOB);

CREATE CONTEXT sequence_check USING sec_mgr.seq_mgr;

CREATE OR REPLACE PACKAGE seq_mgr
AS
  PROCEDURE set_it (p_object IN VARCHAR2);

  PROCEDURE CLEAR;
END;
/

CREATE OR REPLACE PACKAGE BODY seq_mgr
AS
  PROCEDURE set_it (p_object IN VARCHAR2)
  AS
  BEGIN
    DBMS_SESSION.set_context
       (namespace    => 'sequence_check',
        ATTRIBUTE    => 'object_path',
        VALUE        =>    SYS_CONTEXT
                                ('sequence_check',
                                 'object_path')
                        || p_object
                        || ':');
  END;

  PROCEDURE CLEAR
  AS
  BEGIN
    DBMS_SESSION.clear_context
                  (namespace    => 'sequence_check',
                   ATTRIBUTE    => 'object_path');
  END;
END;
/

CREATE OR REPLACE FUNCTION table_pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  seq_mgr.set_it ('TABLE');
  INSERT INTO access_tab
       VALUES (SYS_CONTEXT ('userenv',
                            'session_user'),
               SYSDATE,
               SYS_CONTEXT ('userenv',
                            'current_sql'),
               SYS_CONTEXT ('userenv',
                            'audited_cursorid'),
               SYS_CONTEXT ('sequence_check',
                            'object_path'));
  COMMIT;
  seq_mgr.CLEAR;
  RETURN NULL;
END;
/

CREATE OR REPLACE FUNCTION synonym_pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  seq_mgr.set_it ('SYNONYM');
  RETURN NULL;
END;
/

CREATE OR REPLACE FUNCTION view_pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  seq_mgr.set_it ('VIEW');
  RETURN NULL;
END;
/

BEGIN
  DBMS_RLS.add_policy
       (object_schema      => 'SEC_MGR',
        object_name        => 'PEOPLE',
        policy_name        => 'PEOPLE_TRACE',
        function_schema    => 'SEC_MGR',
        policy_function    => 'TABLE_PRED_FUNCTION',
        statement_types    => 'SELECT');
  DBMS_RLS.add_policy
      (object_schema      => 'SEC_MGR',
       object_name        => 'PEOPLE_SYN',
       policy_name        => 'PEOPLE_SYN_TRACE',
       function_schema    => 'SEC_MGR',
       policy_function    => 'SYNONYM_PRED_FUNCTION',
       statement_types    => 'SELECT');
  DBMS_RLS.add_policy
         (object_schema      => 'SEC_MGR',
          object_name        => 'PEOPLE_VIEW',
          policy_name        => 'PEOPLE_VIEW_TRACE',
          function_schema    => 'SEC_MGR',
          policy_function    => 'VIEW_PRED_FUNCTION',
          statement_types    => 'SELECT');
END;
/

SELECT ename
  FROM people
 WHERE ROWNUM <= 1;
SELECT MAX (sal)
  FROM people_syn;
SELECT COUNT (*)
  FROM people_view;
COL "Query" format a45
COL "Path" format  a30
SELECT DISTINCT audit_id,
                TO_CHAR (sql_text) "Query",
                TO_CHAR (trace_route) "Path"
           FROM access_tab
       ORDER BY audit_id ASC;

SELECT DISTINCT TO_CHAR (sql_text) "Query",
                TO_CHAR (trace_route) "Path"
           FROM access_tab;

CREATE OR REPLACE FUNCTION table_pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
  l_path        VARCHAR2 (2000)
    := SYS_CONTEXT ('sequence_check',
                    'object_path');
  l_return_val  VARCHAR2 (20)   := '1=0';
BEGIN
  IF (INSTR (l_path, 'VIEW') > 0)
  THEN
    l_return_val := '1=1';
  END IF;

  seq_mgr.CLEAR;
  RETURN l_return_val;
END;
/

BEGIN
  DBMS_RLS.drop_policy
                   (object_schema    => 'SEC_MGR',
                    object_name      => 'PEOPLE',
                    policy_name      => 'PEOPLE_TRACE');
  DBMS_RLS.add_policy
        (object_schema      => 'SEC_MGR',
         object_name        => 'PEOPLE',
         policy_name        => 'PEOPLE_TRACE',
         function_schema    => 'SEC_MGR',
         policy_function    => 'TABLE_PRED_FUNCTION',
         statement_types    => 'SELECT');
END;
/

SELECT COUNT (*)
  FROM people;
SELECT COUNT (*)
  FROM people_view;

SPOOL off
/*
drop table access_tab;
drop synonym people_syn;
drop view people_view;
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE_SYN',
                  POLICY_NAME       => 'PEOPLE_SYN_TRACE');
end;
/
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_TRACE');
end;
/
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE_VIEW',
                  POLICY_NAME       => 'PEOPLE_VIEW_TRACE');
end;
/
drop context sequence_check;
drop table people;
*/
EDIT seq.log
