------------------ Views for RLS --------------
conn sec_mgr/sec_mgr

drop context people_ctx;
create context people_ctx using sec_mgr.people_ctx_mgr;
CREATE OR REPLACE
PROCEDURE people_ctx_mgr
AS 
BEGIN
  for rec in 
    (select deptno 
       from people
     where ename = USER)
   LOOP
     DBMS_SESSION.set_context (namespace => 'people_ctx', 
                               attribute => 'deptno', 
                               value     => rec.deptno);  
   END LOOP; 
END;
/
CREATE OR REPLACE 
TRIGGER set_deptno_on_logon
AFTER LOGON ON DATABASE
BEGIN
  sec_mgr.people_ctx_mgr;
END;
/

conn scott/tiger
select sys_context('people_ctx','deptno') from dual;

conn sec_mgr/sec_mgr

drop table people;
create table people as select * from scott.emp;
grant all on people to scott;

conn sys
grant execute on DBMS_RLS to sec_mgr;

conn sec_mgr/sec_mgr
CREATE OR REPLACE 
FUNCTION Dept_Only
           (p_schema in varchar2 default NULL,
            p_object in varchar2 default NULL)
RETURN varchar2
AS
BEGIN
  return 
    'deptno = sys_context(''people_ctx'',''deptno'')';
END;
/
select Dept_Only from dual;


begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_IU');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_IU',
                  FUNCTION_SCHEMA   => 'SEC_MGR',
                  POLICY_FUNCTION   => 'Dept_Only',
                  STATEMENT_TYPES   => 'INSERT,UPDATE',
                  UPDATE_CHECK      =>  TRUE);
end;
/

conn scott/tiger
create synonym people for sec_mgr.people;
select ename, deptno 
  from people 
where ename < 'C';

update people
  set ename = 'GRIZZLY'
where ename = 'ADAMS';

update people
  set ename = 'Bozo'
where ename = 'BLAKE';

insert into people (empno, ename, sal, deptno)
  values (1245, 'KNOX','3000', 20);
insert into people (empno, ename, sal, deptno)
  values (1234, 'ELLISON','2000', 30);
rollback;



-- deletes ---
conn sec_mgr/sec_mgr
CREATE OR REPLACE 
FUNCTION User_Only
           (p_schema in varchar2 default NULL,
            p_object in varchar2 default NULL)
RETURN varchar2
AS
BEGIN
  return 
    'ename = sys_context(''userenv'',''session_user'')';
END;
/

begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_DEL');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_DEL',
                  FUNCTION_SCHEMA   => 'SEC_MGR',
                  POLICY_FUNCTION   => 'User_Only',
                  STATEMENT_TYPES   => 'DELETE');
end;
/

conn scott/tiger
select count(*) from people;
delete from people;
select * from people 
  where ename = 'SCOTT';
rollback;

-- col sens --
conn sec_mgr/sec_mgr
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_SEL_SAL');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_SEL_SAL',
                  FUNCTION_SCHEMA   => 'SEC_MGR',
                  POLICY_FUNCTION   => 'User_Only',
                  STATEMENT_TYPES   => 'SELECT',
                  SEC_RELEVANT_COLS => 'SAL');
end;
/

conn scott/tiger
select count(ename) from people;
select count(sal) from people;

select ename from people;
select ename, sal from people;

-- NULL values --
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE$',
                  POLICY_NAME       => 'PEOPLE_SEL_SAL');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA         => 'SEC_MGR',
                  OBJECT_NAME           => 'PEOPLE$',
                  POLICY_NAME           => 'PEOPLE_SEL_SAL',
                  FUNCTION_SCHEMA       => 'SEC_MGR',
                  POLICY_FUNCTION       => 'User_Only',
                  STATEMENT_TYPES       => 'SELECT',
                  SEC_RELEVANT_COLS     => 'SAL');
end;
/

begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA         => 'SEC_MGR',
                  OBJECT_NAME           => 'PEOPLE$',
                  POLICY_NAME           => 'PEOPLE_SEL_SAL',
                  FUNCTION_SCHEMA       => 'SEC_MGR',
                  POLICY_FUNCTION       => 'User_Only',
                  STATEMENT_TYPES       => 'SELECT',
                  SEC_RELEVANT_COLS     => 'SAL',
                  SEC_RELEVANT_COLS_OPT => DBMS_RLS.ALL_ROWS);
end;
/

-- hacked query result --
CREATE OR REPLACE 
VIEW PEOPLE
AS 
select ename, deptno, job,
       decode(upper(ename),USER,sal,NULL) sal
  from sec_mgr.people;
select ename, sal, deptno from people;

-------- element level security

CREATE OR REPLACE 
FUNCTION User_Only
           (p_schema in varchar2 default NULL,
            p_object in varchar2 default NULL)
RETURN varchar2
AS
BEGIN
  if (p_object = 'PEOPLE_CLS')
  then  
    return 
      'username = sys_context(''userenv'',''session_user'')';
  else 
    return 
      'ename = sys_context(''userenv'',''session_user'')';
  end if;
END;
/
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE_CLS',
                  POLICY_NAME       => 'PEOPLE_CLS_U');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE_CLS',
                  POLICY_NAME       => 'PEOPLE_CLS_U',
                  FUNCTION_SCHEMA   => 'SEC_MGR',
                  POLICY_FUNCTION   => 'User_Only',
                  STATEMENT_TYPES   => 'UPDATE');
end;
/

conn scott/tiger
update sec_mgr.people_cls
  set friendly_name = 'SCOTTIE';
select username, friendly_name from sec_mgr.people_cls;
rollback;






-- query is re-written as --
select * from (select ename, sal from people)
  where ename = USER;

  
-- *******************BUG************
begin
  DBMS_FGA.DROP_POLICY(object_schema=>   'SEC_MGR', 
                      object_name=>     'PEOPLE', 
                      policy_name=>     'FGA_SAL');
end;
/
begin
  DBMS_FGA.ADD_POLICY(object_schema=>   'SEC_MGR', 
                      object_name=>     'PEOPLE', 
                      policy_name=>     'FGA_SAL',
                      audit_condition=> 'ENAME != USER',
                      audit_column =>   'SAL');
end;
/

begin
  DBMS_FGA.DROP_POLICY(object_schema=>   'SEC_MGR', 
                      object_name=>     'PEOPLE', 
                      policy_name=>     'FGA_SAL');
end;
/
begin
  DBMS_FGA.ADD_POLICY(object_schema=>   'SEC_MGR', 
                      object_name=>     'PEOPLE', 
                      policy_name=>     'FGA_SAL',
                      audit_column =>   'SAL');
end;
/
-- add vpd to deptno --
begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_SEL');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA         => 'SEC_MGR',
                  OBJECT_NAME           => 'PEOPLE',
                  POLICY_NAME           => 'PEOPLE_SEL',
                  FUNCTION_SCHEMA       => 'SEC_MGR',
                  POLICY_FUNCTION       => 'User_Only',
                  STATEMENT_TYPES       => 'SELECT',
                  SEC_RELEVANT_COLS     => 'MGR');
end;
/


exec print_table('select * from sys.dba_fga_audit_trail order by timestamp asc');

conn scott/tiger
select ename from people;

----------- performance
conn sys/oracle as sysdba
CREATE OR REPLACE 
PROCEDURE sleep (p_time in number)
AS 
BEGIN
  dbms_lock.sleep(p_time);
END;
/
create public synonym sleep for sleep;
grant execute on sleep to sec_mgr;

conn sec_mgr/sec_mgr
CREATE OR REPLACE function pred_function
       ( p_schema in varchar2 default NULL,
         p_object in varchar2 default NULL)
RETURN varchar2
AS
BEGIN
  sleep(2);
  return null;
END;
/

begin
  DBMS_RLS.DROP_POLICY ( 
                  OBJECT_SCHEMA     => 'SEC_MGR',
                  OBJECT_NAME       => 'PEOPLE',
                  POLICY_NAME       => 'PEOPLE_SEL_CACHE');
end;
/
begin
  DBMS_RLS.ADD_POLICY ( 
                  OBJECT_SCHEMA         => 'SEC_MGR',
                  OBJECT_NAME           => 'PEOPLE',
                  POLICY_NAME           => 'PEOPLE_SEL_CACHE',
                  FUNCTION_SCHEMA       => 'SEC_MGR',
                  POLICY_FUNCTION       => 'PRED_FUNCTION',
                  STATEMENT_TYPES       => 'SELECT',
                  POLICY_TYPE           =>  DBMS_RLS.STATIC);
end;
/

conn scott/tiger
set timing on
select count(*) from people;
select count(*) from people;

