spool vpd_groups.log
/******************************************
          IMPORTANT
The following example does NOT work with 9.2.0.1.0
of the database.  It has been verified to work with
9.0.1.X

The package implementing the driving context must correctly validate the application which 
is being used. Although the database always ensures that the package implementing the driving 
context sets context attributes (by checking the call stack), this fact cannot protect 
against poor or inadequate validation within the package.

******************************************/

conn system/manager

-- Begin by creating a namespace for the driving context.
create context APPS_CTX using sec_mgr.apps_security_init;

conn sec_mgr/sec_mgr
-- Create the package that administers the driving context
CREATE OR REPLACE PACKAGE apps_security_init
AS
   PROCEDURE set_attributes (p_policy in varchar2);
   PROCEDURE clear_attributes;
END;
/


CREATE OR REPLACE PACKAGE BODY apps_security_init
AS
   PROCEDURE set_attributes (p_policy in varchar2)
   AS
   BEGIN
     -- this sets for all sessions of user sec_mgr with client ID set    
     DBMS_SESSION.set_context (namespace => 'APPS_CTX', 
                               attribute => 'ACTIVE_APPS', 
                               value     =>  p_policy
                               );

   END;

   PROCEDURE clear_attributes
   AS
   BEGIN
     dbms_session.clear_context(namespace=>'APPS_CTX', client_id=>NULL, attribute=>'ACTIVE_APPS');
   END;
  
END;
/

grant execute on apps_security_init to public;

-- Create a security function for hack; more about this later
create or replace function pred_function_hack
( p_schema in varchar2, p_object in varchar2 )
return varchar2
as
begin
  return NULL;
end;
/

-- Create a default security function
create or replace function pred_function_default
( p_schema in varchar2, p_object in varchar2 )
return varchar2
as
begin
  return 'sal > 2000';
end;
/

-- Create a security function to return dept 10 records
create or replace function pred_function_10
( p_schema in varchar2, p_object in varchar2 )
return varchar2
as
begin
  return 'deptno = 10';
end;
/

-- Create a security function to return dept 20 records
create or replace function pred_function_20
( p_schema in varchar2, p_object in varchar2 )
return varchar2
as
begin
  return 'deptno = 20';
end;
/

conn system/manager

  -- add default policy
BEGIN
  DBMS_RLS.ADD_GROUPED_POLICY(object_schema=>'SCOTT',
                              object_name =>'EMP',
                              policy_group => 'SYS_DEFAULT',
                              policy_name => 'DEFAULT_EMP_POLICY',
                              function_schema =>'sec_mgr',
                              policy_function =>'pred_function_default',
                              statement_types => 'SELECT'
                              );
END;
/

-- define mapping of application context and FGAC target object(s)
BEGIN
  DBMS_RLS.ADD_POLICY_CONTEXT(object_schema => 'SCOTT',
                              object_name   => 'EMP',
                              namespace     => 'APPS_CTX',
                              attribute     => 'ACTIVE_APPS');
  END;
/

-- create a default group to prevent all groups from running
BEGIN
  -- create a ACTIVE_APPS group for APP1
  DBMS_RLS.CREATE_POLICY_GROUP(object_schema => 'SCOTT',
                               object_name   => 'EMP', 
                               policy_group  =>'HACK_DEFAULT');

  -- associate ACTIVE_APPS with group and predicate function
  DBMS_RLS.ADD_GROUPED_POLICY(object_schema   =>'SCOTT',
                              object_name     =>'EMP',
                              policy_group    => 'HACK_DEFAULT',
                              policy_name     =>  'DEFAULT_POLICY',
                              function_schema =>'sec_mgr',
                              policy_function =>'pred_function_hack',
                              statement_types => 'SELECT'
                              );
END;
/


BEGIN
  -- create a ACTIVE_APPS group for APP1
  DBMS_RLS.CREATE_POLICY_GROUP(object_schema => 'SCOTT',
                               object_name   => 'EMP', 
                               policy_group=>'APP1_DEPT10');

  -- associate ACTIVE_APPS with group and predicate function
  DBMS_RLS.ADD_GROUPED_POLICY(object_schema=>'SCOTT',
                              object_name =>'EMP',
                              policy_group => 'APP1_DEPT10',
                              policy_name => 'APP1_POLICY',
                              function_schema =>'sec_mgr',
                              policy_function =>'pred_function_10',
                              statement_types => 'SELECT'
                              );
END;
/

BEGIN
  -- create a ACTIVE_APPS group for APP2
  DBMS_RLS.CREATE_POLICY_GROUP(object_schema => 'SCOTT',
                               object_name   => 'EMP', 
                               policy_group=>'APP2_DEPT20');

  -- associate ACTIVE_APPS with group and predicate function
  DBMS_RLS.ADD_GROUPED_POLICY(object_schema=>'SCOTT',
                              object_name =>'EMP',
                              policy_group => 'APP2_DEPT20',
                              policy_name => 'APP2_POLICY',
                              function_schema =>'sec_mgr',
                              policy_function =>'pred_function_20',
                              statement_types => 'SELECT'
                              );
END;
/



-- show value of driving context
select sys_context('APPS_CTX','ACTIVE_APPS') from dual;
-- show results of basic query
select ename, sal, deptno from scott.emp;
exec print_table ('select * from v$vpd_policy where object_name = ''EMP''');

-- set default hack to get the actual default going
exec sec_mgr.apps_security_init.set_attributes('HACK_DEFAULT');
select sys_context('APPS_CTX','ACTIVE_APPS') from dual;
select ename, sal, deptno from scott.emp;

-- set driving context to app1, NOTE: Default is still working
exec sec_mgr.apps_security_init.set_attributes('APP1_DEPT10');
select sys_context('APPS_CTX','ACTIVE_APPS') from dual;
select ename, sal, deptno from scott.emp;

-- set driving context to app2
exec sec_mgr.apps_security_init.set_attributes('APP2_DEPT20');
select sys_context('APPS_CTX','ACTIVE_APPS') from dual;
select ename, sal, deptno from scott.emp;

spool off
conn system/manager
exec sec_mgr.apps_security_init.clear_attributes;
exec dbms_rls.drop_grouped_policy(object_schema=>'SCOTT', object_name =>'EMP', policy_group => 'SYS_DEFAULT', policy_name => 'default_emp_policy');
exec dbms_rls.drop_grouped_policy(object_schema=>'SCOTT', object_name =>'EMP', policy_group => 'HACK_DEFAULT', policy_name => 'DEFAULT_POLICY');
exec dbms_rls.delete_policy_group(object_schema => 'SCOTT', object_name   => 'EMP', policy_group=>'HACK_DEFAULT');
exec dbms_rls.drop_grouped_policy(object_schema=>'SCOTT', object_name =>'EMP', policy_group => 'APP1_DEPT10', policy_name => 'APP1_POLICY');
exec dbms_rls.delete_policy_group(object_schema => 'SCOTT', object_name   => 'EMP', policy_group=>'APP1_DEPT10');
exec dbms_rls.drop_grouped_policy(object_schema=>'SCOTT', object_name =>'EMP', policy_group => 'APP2_DEPT20', policy_name => 'APP2_POLICY');
exec dbms_rls.delete_policy_group(object_schema => 'SCOTT', object_name   => 'EMP', policy_group=>'APP2_DEPT20');
exec dbms_rls.drop_policy_context(object_schema => 'SCOTT', object_name   => 'EMP', namespace     => 'APPS_CTX', attribute     => 'ACTIVE_APPS');

drop context APPS_CTX;
drop package sec_mgr.apps_security_init;
drop function sec_mgr.pred_function_default;
drop function sec_mgr.pred_function_10;
drop function sec_mgr.pred_function_20;
edit vpd_groups.log
exit
