-------Quick start -----------
CONN sec_mgr/oracle10g
-- Create policy function.

CREATE OR REPLACE FUNCTION no_dept10 (
  p_schema  IN  VARCHAR2,
  p_object  IN  VARCHAR2)
  RETURN VARCHAR2
AS
BEGIN
  RETURN 'deptno != 10';
END;
/

------------ add the policy ---------
-- apply RLS policy to table

BEGIN
  DBMS_RLS.add_policy
                  (object_schema      => 'SCOTT',
                   object_name        => 'EMP',
                   policy_name        => 'quickstart',
                   policy_function    => 'no_dept10');
END;
/

CONN scott/tiger
-- Show department numbers.
-- There should be no department 10.
SELECT DISTINCT deptno FROM emp;
	
CONN sec_mgr/oracle10g
-- change policy implementation to 
-- remove all records for the SYSTEM user
CREATE OR REPLACE FUNCTION no_dept10 (
  p_schema  IN  VARCHAR2,
  p_object  IN  VARCHAR2)
  RETURN VARCHAR2
AS
BEGIN
  RETURN 'USER != ''SYSTEM''';
END;
/
-- Test by counting records as SCOTT
-- then as SYSTEM
CONN scott/tiger
SELECT COUNT(*) Total_Records FROM emp;
CONN system/manager
SELECT COUNT(*) Total_Records FROM scott.emp;

------------------ Views for RLS --------------
CONN scott/tiger
DROP TABLE people;
CREATE TABLE people
AS SELECT ename username, job, sal salary, deptno
  FROM emp;
GRANT SELECT ON people TO sec_mgr;
CONN sec_mgr/oracle10g

drop table lookup_dept;
-- Create table to populate application context values
CREATE TABLE lookup_dept
AS SELECT username, deptno FROM scott.people;

DROP CONTEXT people_ctx;
-- Create namespace for application context
CREATE CONTEXT people_ctx USING sec_mgr.people_ctx_mgr;
-- Create namespace manager program for modifying context.
CREATE OR REPLACE PACKAGE people_ctx_mgr
AS
  PROCEDURE set_deptno;
  PROCEDURE clear_deptno;
END;
/

CREATE OR REPLACE PACKAGE BODY people_ctx_mgr
AS
-----------------------------------------
  PROCEDURE set_deptno
  AS
    l_deptno  NUMBER;
  BEGIN
    SELECT deptno
      INTO l_deptno
      FROM lookup_dept
     WHERE username =
             SYS_CONTEXT ('userenv',
                          'session_user');
    DBMS_SESSION.set_context
                       (namespace    => 'people_ctx',
                        ATTRIBUTE    => 'deptno',
                        VALUE        => l_deptno);
  END set_deptno;
-----------------------------------------
  PROCEDURE clear_deptno
  AS
  BEGIN
    DBMS_SESSION.clear_context
                      (namespace    => 'people_ctx',
                       ATTRIBUTE    => 'deptno');
  END clear_deptno;
-----------------------------------------
END people_ctx_mgr;
/
-- do NOT have to grant execute on namespace manager

CREATE OR REPLACE TRIGGER set_user_deptno
  AFTER LOGON ON DATABASE
BEGIN
  sec_mgr.people_ctx_mgr.set_deptno;
EXCEPTION
  WHEN NO_DATA_FOUND
  THEN
    -- user is not in our table
    -- so the value will not be set.
    -- If exception is not handled, then some users
    -- may be unable to log on
    NULL;
END;
/


CONN scott/tiger
COL deptno format a8
SELECT SYS_CONTEXT ('people_ctx',
                    'deptno') deptno
  FROM DUAL;

CONN sec_mgr/oracle10g
-- Create policy function.
CREATE OR REPLACE FUNCTION dept_only (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  RETURN 'deptno = sys_context(''people_ctx'',''deptno'')';
END;
/

-- Test policy function.
col predicate format a50
SELECT dept_only predicate FROM DUAL;


------------ add the policy ---------
-- apply RLS policy to table
BEGIN
  DBMS_RLS.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'PEOPLE',
              policy_name        => 'PEOPLE_IU',
              function_schema    => 'SEC_MGR',
              policy_function    => 'Dept_Only',
              statement_types    => 'INSERT,UPDATE',
              update_check       => TRUE);
END;
/

-------- test the policy ---------
CONN scott/tiger
SELECT username, deptno
  FROM people
 WHERE username < 'C';

-- Update to dept 20 user
-- This should succeed.
UPDATE people
   SET username = 'GRIZZLY'
 WHERE username = 'ADAMS';
-- Update to dept 30 user
-- This should not update anything.
UPDATE people
   SET username = 'BOZO'
 WHERE username = 'BLAKE';

-- This insert should work as deptno = 20
INSERT INTO people
            (username, job, salary, deptno)
     VALUES ('KNOX', 'Clerk', '3000', 20);
-- This insert should not work 
INSERT INTO people
            (username, job, salary, deptno)
     VALUES ('ELLISON', 'CEO','90000', 30);

conn sec_mgr/oracle10g
-------------- DELETE policy -------------
CREATE OR REPLACE FUNCTION user_only (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  RETURN 'username = sys_context(''userenv'',''session_user'')';
END;
/
------------ add the policy ---------
-- apply delete policy to table
BEGIN
  DBMS_RLS.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'PEOPLE',
              policy_name        => 'People_Del',
              function_schema    => 'SEC_MGR',
              policy_function    => 'user_only',
              statement_types    => 'DELETE');
END;
/

-- test deletes --
conn scott/tiger
DELETE FROM people;
SELECT * FROM people
 WHERE username = 'SCOTT';


------------ RLS Layer section -----
CONN system/manager
-- user has SELECT ANY TABLE so he can see all records
SELECT COUNT(*) FROM scott.people;
-- user cannot delete records as RLS prevents this
DELETE FROM scott.people;

conn sec_mgr/oracle10g
----no records------
-- Create a function to be used with RLS
-- that will always eliminate all records.
CREATE OR REPLACE FUNCTION no_records (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  RETURN '1=0';
END;
/

CONN scott/tiger
DROP TABLE people_ro;
CREATE TABLE people_ro AS SELECT * FROM emp;
conn sec_mgr/oracle10g
BEGIN
  DBMS_RLS.add_policy
      (object_schema      => 'SCOTT',
       object_name        => 'PEOPLE_RO',
       policy_name        => 'PEOPLE_RO_IUD',
       function_schema    => 'SEC_MGR',
       policy_function    => 'No_Records',
       statement_types    => 'INSERT,UPDATE,DELETE',
       update_check       => TRUE);
END;
/


CONN scott/tiger
-- User can read all records
SELECT COUNT (*) FROM people_ro;
-- Cannot update any records
UPDATE people_ro
   SET ename = NULL;
-- Cannot delete records
DELETE FROM people_ro;
-- Cannot insert new records
INSERT INTO people_ro (ename) VALUES ('KNOX');
-- reset
BEGIN
  DBMS_RLS.drop_policy
                  (object_schema    => 'SCOTT',
                   object_name      => 'PEOPLE_RO',
                   policy_name      => 'PEOPLE_RO_IUD');
END;
/


---------- RLS exemptions ----------
CONN system/manager
-- Show system is affected by RLS policy.
-- No records should be deleted.
DELETE FROM scott.people_ro;
-- grant privilege to by-pass RLS policies
GRANT EXEMPT ACCESS POLICY TO SYSTEM;
-- Show system is no longer affected by RLS policy.
-- All records should be deleted.
DELETE FROM scott.people_ro;
ROLLBACK ;
-- Show users that are exempt from RLS policies
SELECT grantee
  FROM dba_sys_privs
 WHERE PRIVILEGE = 'EXEMPT ACCESS POLICY';

CONN sec_mgr/oracle10g
-- Audit the exempt policy system privilege
AUDIT EXEMPT ACCESS POLICY BY ACCESS;
-- Test audit by exercising the system privilege
CONN system/manager
DELETE FROM scott.people_ro;
-- Rollback will not erase the audit record
ROLLBACK ;
CONN sec_mgr/oracle10g
-- show audited operation
BEGIN
  FOR rec IN (SELECT *
                FROM dba_audit_trail)
  LOOP
    DBMS_OUTPUT.put_line ('-------------------------');
    DBMS_OUTPUT.put_line ('Who:   ' || rec.username);
    DBMS_OUTPUT.put_line (   'What:  '
                          || rec.action_name
                          || ' on '
                          || rec.owner
                          || '.'
                          || rec.obj_name);
    DBMS_OUTPUT.put_line (   'When:  '
                          || TO_CHAR
                                  (rec.TIMESTAMP,
                                   'MM/DD HH24:MI'));
    DBMS_OUTPUT.put_line ('How:   "' || rec.sql_text || '"');
    DBMS_OUTPUT.put_line ('Using: ' || rec.priv_used);
  END LOOP;
END;
/
 
COL who format a6
COL what format a25
COL when format a12
COL how format a27
COL using format a20
SELECT username "Who",
          action_name
       || ' on '
       || owner
       || '.'
       || obj_name "What",
       TO_CHAR (TIMESTAMP, 'MM/DD HH24:MI')
                                           "When",
       sql_text "How",
       priv_used "Using"
  FROM dba_audit_trail;
-- disable audit and reset logs
NOAUDIT EXEMPT ACCESS POLICY;
CONN system/manager
DELETE FROM aud$;
COMMIT ;
REVOKE EXEMPT ACCESS POLICY FROM SYSTEM;


-------- Debugging ---------------
CONN scott/tiger
-- Create a dependency. This table will
-- be called by the RLS policy function.
CREATE TABLE t AS SELECT * FROM DUAL;
-- Create policy function. Function
-- is dependent on table T.
CREATE OR REPLACE FUNCTION pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
  l_total_recs  NUMBER;
BEGIN
  SELECT COUNT (*)
    INTO l_total_recs
    FROM t;
  RETURN '1 <= ' || l_total_recs;
END;
/
CONN sec_mgr/oracle10g
-- Add RLS policy to EMP table;
BEGIN
  DBMS_RLS.add_policy
              (object_schema      => 'SCOTT',
               object_name        => 'EMP',
               policy_name        => 'debug',
               function_schema    => 'SCOTT',
               policy_function    => 'pred_function');
END;
/

CONN scott/tiger
-- Everything initially works fine
SELECT COUNT(*) FROM emp;
-- This drop breaks the policy function
DROP TABLE t;
-- Policy function is invalid and will not recompile;
SELECT COUNT(*) FROM emp;
ALTER FUNCTION pred_function COMPILE;
SHOW errors

-- Recover table
FLASHBACK TABLE t TO BEFORE DROP;
SELECT COUNT(*) FROM emp;

-- User dynamic SQL and exception handling
-- to mask policy function errors
CREATE OR REPLACE FUNCTION pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
  l_total_recs  NUMBER;
BEGIN
  -- Dynamic SQL hides the dependency on table T
  EXECUTE IMMEDIATE 'SELECT COUNT (*) FROM t'
               INTO l_total_recs;
  RETURN '1 <= ' || l_total_recs;
EXCEPTION
  WHEN OTHERS
  THEN
    -- Fail Secure: remove all rows
    RETURN '1=0';
END;
/
SELECT COUNT(*) FROM emp;
-- This drop breaks the policy function
DROP TABLE t;
-- Policy fails secure. No records are displayed or
-- exception messages given to user.
SELECT COUNT(*) FROM emp;

-- Recover table
FLASHBACK TABLE t TO BEFORE DROP;
SELECT COUNT(*) FROM emp;


-- reset; remove policy
BEGIN
  DBMS_RLS.drop_policy
                  (object_schema    => 'SCOTT',
                   object_name      => 'EMP',
                   policy_name      => 'debug');
END;
/

--------- Invalid SQL --------------
-- Create policy function that returns
-- records just for the user.
CREATE OR REPLACE FUNCTION pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  -- Restricting records by mapping ENAME
  -- to authenticated database user's name
  RETURN 'name = user';
END;
/
SELECT COUNT(*) FROM emp;

col "Policy Predicate" format a20
SELECT pred_function "Policy Predicate" FROM DUAL;

----- trace files --------
CONN sec_mgr/oracle10g
-- Create view to display session's current trace file
CREATE OR REPLACE VIEW get_trace_filename
AS
  SELECT    c.VALUE
         || decode(instr(c.value,'\'),0,'/','\')
         || INSTANCE
         || '_ora_'
         || LTRIM (TO_CHAR (a.spid, 'fm99999'))
         || '.trc' filename
    FROM v$process a,
         v$session b,
         v$parameter c,
         v$thread c
   WHERE a.addr = b.paddr
     AND b.audsid = USERENV ('sessionid')
     AND c.NAME = 'user_dump_dest';
GRANT SELECT ON get_trace_filename TO PUBLIC;
CREATE PUBLIC SYNONYM get_trace_filename 
                  FOR get_trace_filename;

CONN scott/tiger
-- Enable tracing
ALTER SESSION SET EVENTS
  '10730 trace name context forever, level 12';
-- Reproduce error.
-- Issue query; note only selecting ENAME
SELECT ename FROM emp;
-- view trace file
COL filename format a50
SELECT * FROM get_trace_filename;

-- fix the policy function
-- Create policy function that returns
-- records just for the user.
CREATE OR REPLACE FUNCTION pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  -- Restricting records by mapping ENAME
  -- to authenticated database user's name
  RETURN 'ename = user';
END;
/
SELECT ename FROM emp;

------------- V$VPD_POLICY
CONN system/manager
-- clear buffer to simulate a fresh database
ALTER SYSTEM FLUSH SHARED_POOL;
CONN sec_mgr/oracle10g
-- nothing there
SELECT * FROM v$vpd_policy;
CONN scott/tiger
-- this will seed the v$vpd_policy view
SELECT COUNT (*) FROM emp;
CONN sec_mgr/oracle10g
-- show VPD Policy results
COL object_owner format a12
COL object_name format a12
COL policy format a6
COL sql_fulltext format a26
COL predicate format a12
SELECT object_owner,
       object_name,
       policy,
       sql_fulltext,
       predicate
  FROM v$vpd_policy p, v$sql s
 WHERE p.sql_id = s.sql_id
   AND predicate IS NOT NULL;

----------------- NULL Context Values ----------
CONN scott/tiger
-- Context value is set on logon
COL deptno format a6
SELECT SYS_CONTEXT ('people_ctx',
                    'deptno') deptno
  FROM DUAL;
CONN sec_mgr/oracle10g
-- Current view on table
SELECT * FROM lookup_dept
 WHERE username = 'SCOTT';
-- Add Dept level policy to lookup table.
-- Policy references people_ctx context.
BEGIN
  DBMS_RLS.add_policy
               (object_schema      => 'SEC_MGR',
                object_name        => 'lookup_dept',
                policy_name        => 'lookup_dept_sel',
                policy_function    => 'dept_only',
                statement_types    => 'SELECT');
END;
/
-- Add Dept level policy to EMP table.
-- Policy references people_ctx context.
BEGIN
  DBMS_RLS.add_policy
               (object_schema      => 'SCOTT',
                object_name        => 'EMP',
                policy_name        => 'EMP_UPD',
                policy_function    => 'dept_only',
                statement_types    => 'UPDATE');
END;
/
-- rows are no longer visible since SEC_MGR has not context
SELECT * FROM lookup_dept;

CONN scott/tiger
-- Context no longer exists
COL deptno format A6
SELECT SYS_CONTEXT ('people_ctx',
                    'deptno') deptno
  FROM DUAL;
-- any RLS policies that used the context value will not behave correctly
UPDATE emp
   SET ename = ename;
EXEC sec_mgr.debug_policy('scott','people','people_sel')

-- reset
conn sec_mgr/oracle10g
BEGIN
  DBMS_RLS.drop_policy
                  (object_schema    => 'SEC_MGR',
                   object_name      => 'lookup_dept',
                   policy_name      => 'lookup_dept_sel');
END;
/

------------------------ Column Sensitive ------------
BEGIN
  DBMS_RLS.add_policy
                (object_schema        => 'SCOTT',
                 object_name          => 'PEOPLE',
                 policy_name          => 'people_sel_sal',
                 function_schema      => 'SEC_MGR',
                 policy_function      => 'user_only',
                 statement_types      => 'SELECT',
                 sec_relevant_cols    => 'SALARY');
END;
/

CONN scott/tiger
-- User can see all records. Show first five records only. 
SELECT username FROM people
 WHERE ROWNUM <= 5;
-- Adding the salary column causes the RLS policy to activate
SELECT username, salary FROM people;

CONN sec_mgr/oracle10g
BEGIN
  -- Remove current policy
  DBMS_RLS.drop_policy
                 (object_schema    => 'SCOTT',
                  object_name      => 'PEOPLE',
                  policy_name      => 'people_sel_sal');
  -- Add policy again but now tih SEC_RELEVANT_COLS_OPT
  DBMS_RLS.add_policy
       (object_schema            => 'SCOTT',
        object_name              => 'PEOPLE',
        policy_name              => 'people_sel_sal',
        function_schema          => 'SEC_MGR',
        policy_function          => 'user_only',
        statement_types          => 'SELECT',
        sec_relevant_cols        => 'SALARY',
        SEC_RELEVANT_COLS_OPT    => DBMS_RLS.all_rows);
END;
/
CONN scott/tiger
-- Show outcome of all_rows added as sec_relevant_cols_opt
-- Just showing dept20 records for brevity.
SELECT username, salary FROM people
 WHERE deptno = 20;

-- 
update people set salary=salary;

-- Show that updates are influenced when done as SELECT FOR UPDATE
begin
  for rec in (select * from people for update)
  loop
     update people set salary=rec.salary+1;
     dbms_output.put_line(rec.username);
  end loop;
end;
/


-------------------- PERFORMANCE -------------------
conn sec_mgr/oracle10g
CREATE OR REPLACE FUNCTION dept_only (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  -- Return predicate with value resolved.
  RETURN 'deptno = ' || sys_context('people_ctx','deptno');
END;
/


CREATE OR REPLACE PROCEDURE sleep (p_time IN NUMBER)
AS
BEGIN
  DBMS_LOCK.sleep (p_time);
END;
/

CREATE OR REPLACE FUNCTION user_only (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  -- stall for 2 seconds
  DBMS_LOCK.sleep (2);
  RETURN 'username = sys_context(''userenv'',''session_user'')';
END;
/

conn scott/tiger
set timing on
SELECT username, salary FROM people
 WHERE deptno = 20;

conn sec_mgr/oracle10g
BEGIN
  -- Remove current policy
  DBMS_RLS.drop_policy
                 (object_schema    => 'SCOTT',
                  object_name      => 'PEOPLE',
                  policy_name      => 'people_sel_sal');
  -- Add policy again but now with Caching
  DBMS_RLS.add_policy
      (object_schema            => 'SCOTT',
       object_name              => 'PEOPLE',
       policy_name              => 'people_sel_sal',
       function_schema          => 'SEC_MGR',
       policy_function          => 'user_only',
       statement_types          => 'SELECT',
       sec_relevant_cols        => 'SALARY',
       policy_type              => DBMS_RLS.STATIC);
END;
/

-- Reconnect. Establish a different session for same user
conn scott/tiger
SET timing on
SELECT username, salary FROM people
 WHERE deptno = 20;
-- Connect as a different user
conn system/manager
SET timing on
SELECT username, salary FROM scott.people
 WHERE deptno = 20;

--------- shared static -----------
CONN sec_mgr/oracle10g
-- disable all select policies for EMP, and DEPT
DECLARE
  l_str  VARCHAR2 (100);
BEGIN
  FOR rec IN (SELECT *
                FROM dba_policies
               WHERE object_owner = 'SCOTT'
                 AND object_name IN
                        ('EMP', 'DEPT')
                 AND sel = 'YES')
  LOOP
    l_str :=
         'begin DBMS_RLS.drop_policy(''SCOTT'','''
      || rec.object_name
      || ''','''
      || rec.policy_name
      || '''); end;';
    DBMS_OUTPUT.put_line (l_str);
    EXECUTE IMMEDIATE l_str;
  END LOOP;
END;
/

-- Vpd function, injects 2 second delay
CREATE OR REPLACE FUNCTION pred_function (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  DBMS_LOCK.sleep (2);
  -- return all records
  RETURN NULL;
END;
/

BEGIN
  -- Add policy on EMP table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'EMP',
            policy_name        => 'EMP_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
  -- Add policy on DEPT table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'DEPT',
            policy_name        => 'DEPT_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_static);
END;
/
CONN scott/tiger
SET timing on
-- Query one of the tables
SELECT COUNT (*) FROM emp;
-- run again to see if predicate is cached
/
-- Query a different table that uses same policy function
-- The database should use the cached predicate
SELECT COUNT (*) FROM dept;
-- Query as a different session
-- Cache is not shared across sessions
conn system/manager
SELECT COUNT (*) FROM scott.dept;


--------- shared_context_sensitive -----------
CONN sec_mgr/oracle10g
-- disable all select policies for EMP and DEPT 
DECLARE
  l_str  VARCHAR2 (100);
BEGIN
  FOR rec IN (SELECT *
                FROM dba_policies
               WHERE object_owner = 'SCOTT'
                 AND object_name IN
                        ('EMP', 'DEPT')
                 AND sel = 'YES')
  LOOP
    l_str :=
         'begin DBMS_RLS.drop_policy(''SCOTT'','''
      || rec.object_name
      || ''','''
      || rec.policy_name
      || '''); end;';
    DBMS_OUTPUT.put_line (l_str);
    EXECUTE IMMEDIATE l_str;
  END LOOP;
END;
/
-- add shared_context_sensitive policies
BEGIN
  -- Add policy on EMP table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'EMP',
            policy_name        => 'EMP_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_context_sensitive);
  -- Add policy on DEPT table
  DBMS_RLS.add_policy
           (object_schema      => 'SCOTT',
            object_name        => 'DEPT',
            policy_name        => 'DEPT_SEL_CACHE',
            function_schema    => 'SEC_MGR',
            policy_function    => 'PRED_FUNCTION',
            statement_types    => 'SELECT',
            policy_type        => DBMS_RLS.shared_context_sensitive);
END;
/
-- re-write namespace manager to allow user/application to
-- manipulate context
CREATE OR REPLACE PACKAGE people_ctx_mgr
AS
  PROCEDURE set_deptno;
  PROCEDURE set_deptno(p_deptno in number);
  PROCEDURE clear_deptno;
END;
/

CREATE OR REPLACE PACKAGE BODY people_ctx_mgr
AS
-----------------------------------------
  PROCEDURE set_deptno
  AS
    l_deptno  NUMBER;
  BEGIN
    SELECT deptno
      INTO l_deptno
      FROM lookup_dept
     WHERE username =
             SYS_CONTEXT ('userenv',
                          'session_user');
    DBMS_SESSION.set_context
                       (namespace    => 'people_ctx',
                        ATTRIBUTE    => 'deptno',
                        VALUE        => l_deptno);
  END set_deptno;

-----------------------------------------
  PROCEDURE set_deptno (p_deptno IN NUMBER)
  AS
  BEGIN
    DBMS_SESSION.set_context
                      (namespace    => 'people_ctx',
                       ATTRIBUTE    => 'deptno',
                       VALUE        => p_deptno);
  END set_deptno;

-----------------------------------------
  PROCEDURE clear_deptno
  AS
  BEGIN
    DBMS_SESSION.clear_context
                      (namespace    => 'people_ctx',
                       ATTRIBUTE    => 'deptno');
  END clear_deptno;
-----------------------------------------
END people_ctx_mgr;
/
GRANT EXECUTE ON people_ctx_mgr TO scott;


CONN scott/tiger
SET timing on
-- Query one of the tables. Predicate is not yet cached
SELECT COUNT (*) FROM emp;
-- Query a different table that uses same policy function
-- The database should use the cached predicate
SELECT COUNT (*) FROM dept;
-- note that changing the Client Identifier has no
-- affect on cache
EXEC dbms_session.set_identifier('Some Value');
SELECT COUNT (*) FROM emp;
-- Execute namespace manager procedure.
-- This switches a context which tells database to
-- not use the cached predicate
EXEC sec_mgr.people_ctx_mgr.set_deptno(10);
SELECT COUNT (*) FROM emp;
-- Query as a different session
-- Cache is not shared across sessions
conn scott/tiger
SELECT COUNT (*) FROM scott.dept;



-------------- view speed test -----------
conn sec_mgr/oracle10g
CREATE OR REPLACE FUNCTION owner_admin (
  p_schema  IN  VARCHAR2 DEFAULT NULL,
  p_object  IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
BEGIN
  IF (SYS_CONTEXT ('userenv', 'isdba') = 'TRUE')
  THEN
    RETURN NULL;              -- returns all rows
  ELSE
    RETURN 'OWNER = USER';
  END IF;
END;
/

CREATE OR REPLACE VIEW big_vpd_view
AS
  SELECT * FROM big_tab;

BEGIN
  DBMS_RLS.add_policy
             (object_name        => 'BIG_VPD_VIEW',
              policy_name        => 'BIG_VPD_VIEW_SIUD',
              policy_function    => 'owner_admin');
END;
/

SET timing on
-- time with security built into SQL
SELECT COUNT (*)
  FROM big_tab
 WHERE 1 = DECODE (owner, USER, 1, 0)
    OR SYS_CONTEXT ('userenv', 'isdba') = 'TRUE';
-- time with RLS built into view
SELECT COUNT (*)
  FROM big_view;
-- time with VPD
SELECT COUNT (*)
  FROM big_vpd_view;
