/*************
 TRUSTED PROGRAM VERSUS DR VIEW
 
 if auth given to schema, and schema creates view,
 do OLS priviegles convey with execute privileges?
*************/

COL "Read Label" format a30
COL "Write Label" format a30
COL "Label" format a12

CONN scott/tiger
SELECT sa_session.read_label ('ESBD')
                                     "Read Label",
       sa_session.write_label ('ESBD')
                                    "Write Label"
  FROM DUAL;

CREATE OR REPLACE VIEW total_announcements
AS
  SELECT   label_to_char (ROWLABEL) label,
           COUNT (ROWLABEL) total_messages
      FROM announcements
  GROUP BY ROWLABEL;

SELECT *
  FROM total_announcements;

GRANT SELECT ON total_announcements TO blake;
GRANT SELECT ON announcements TO blake;

CONN blake/blake
-- blake has no authorizations
SELECT sa_session.read_label ('ESBD')
                                     "Read Label",
       sa_session.write_label ('ESBD')
                                    "Write Label"
  FROM DUAL;
SELECT COUNT (*)
  FROM scott.announcements;
SELECT *
  FROM scott.total_announcements;


CONN scott/tiger
-- create function to calculate total

CREATE OR REPLACE FUNCTION label_count (
  p_rowlabel  IN  NUMBER)
  RETURN NUMBER
AS
  l_total  NUMBER := 0;
BEGIN
  SELECT COUNT (ROWLABEL)
    INTO l_total
    FROM announcements
   WHERE ROWLABEL = p_rowlabel;
  RETURN l_total;
END;
/

-- grant privileges to function
CONN sec_mgr/sec_mgr

BEGIN
  sa_user_admin.set_prog_privs
             (policy_name          => 'ESBD',
              schema_name          => 'SCOTT',
              program_unit_name    => 'label_count',
              PRIVILEGES           => 'READ');
END;
/

CONN scott/tiger
-- update view to call function
CREATE OR REPLACE VIEW total_announcements
AS
  SELECT   label_to_char (ROWLABEL) label,
           label_count (ROWLABEL) total_messages
      FROM announcements
  GROUP BY ROWLABEL;

SELECT *
  FROM total_announcements;

CONN blake/blake
-- blake has no authorizations
SELECT sa_session.read_label ('ESBD')
                                     "Read Label",
       sa_session.write_label ('ESBD')
                                    "Write Label"
  FROM DUAL;
SELECT COUNT (*)
  FROM scott.announcements;
SELECT *
  FROM scott.total_announcements;
