COL "OLS Label" format a12
COL message format a63
COL "Read Label" format a30
COL "Write Label" format a30

-- page 3-13 dilemma
-- can a user with groups override the write access on comp?
-- need to enforce write control
/***************
If the label has no groups, then the user must have write access on all the
compartments in the label, in order to write the data.

If the label does have groups, and the user has write access to one of the groups,
she only needs read access to the compartments, in order to write the data
****************/
CONN sec_mgr/sec_mgr
EXEC sa_session.set_access_profile('ESBD','ALL_EXECS');
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;
-- turn on write control

BEGIN
  sa_policy_admin.remove_table_policy
                   (policy_name    => 'ESBD',
                    schema_name    => 'SCOTT',
                    table_name     => 'ANNOUNCEMENTS');
  sa_policy_admin.apply_table_policy
    (policy_name      => 'ESBD',
     schema_name      => 'SCOTT',
     table_name       => 'ANNOUNCEMENTS',
     table_options    => 'HIDE,LABEL_DEFAULT,WRITE_CONTROL,READ_CONTROL');
END;
/

-- add one compartment for scott which is read/write

BEGIN
  sa_user_admin.add_compartments
               (policy_name    => 'ESBD',
                user_name      => 'SCOTT',
                comps          => 'DEV',
                access_mode    => sa_utl.read_write,
                in_def         => 'Y',
                in_row         => 'Y');
END;
/

-- add other compartment for scott which is read only
-- we will write to this compartment even though
-- the compartment writes say read only

BEGIN
  sa_user_admin.add_compartments
                (policy_name    => 'ESBD',
                 user_name      => 'SCOTT',
                 comps          => 'SALES',
                 access_mode    => sa_utl.read_only,
                 in_def         => 'Y',
                 in_row         => 'N');
END;
/

-- add groups. if groups exist, then read_write overrides comp write enforcement.

BEGIN
  sa_user_admin.add_groups
               (policy_name    => 'ESBD',
                user_name      => 'SCOTT',
                GROUPS         => 'US',
                access_mode    => sa_utl.read_write,
                in_def         => 'Y',
                in_row         => 'Y');
END;
/

CONN scott/tiger
SELECT sa_session.read_label ('ESBD')
                                     "Read Label",
       sa_session.write_label ('ESBD')
                                    "Write Label"
  FROM DUAL;
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;
-- delete from sales compartment. write access is regulated
-- by groups
DELETE FROM announcements
      WHERE label_to_char (ROWLABEL) LIKE
                                       '%:SALES:%';
SELECT MESSAGE,
       label_to_char (ROWLABEL) "OLS Label"
  FROM scott.announcements;
ROLLBACK ;
