spool ch13.log
-------------- utility function of data_crypto functions-------------
COL Encrypted format a32
SELECT data_crypto.encrypt ('Data',
                            'Key',
                            data_crypto.aes256)
                                        encrypted
  FROM DUAL;

SELECT dbms_crypto.encrypt
         (UTL_RAW.cast_to_raw ('data'),
          data_crypto.aes,
          data_crypto.getkey
                        (   'my encryption key'
                         || ' of some random length',
                         data_crypto.aes))
                                        encrypted
  FROM DUAL;

-------------- simple examples -------------

-- DBMS_CRYPTO simple example
DECLARE
  -- The encryption algorithm that will be used.
  -- Using AES with 128-bit key size on a CBC modifier
  -- and padded using the PKCS#5 padding standard
  l_algorithm  PLS_INTEGER
    :=   dbms_crypto.encrypt_aes128
       + dbms_crypto.chain_cbc
       + dbms_crypto.pad_pkcs5;
  -- The data that will be encrypted
  l_data       VARCHAR2 (4)  := 'DATA';
  -- The key that will be used for the encryption.
  -- Key has to be 16 bytes for AES128 algorithm
  l_key        VARCHAR2 (16) := 'TheEncryptionKey';
  -- Initialization vector. Vector has to be
  -- 16 bytes for AES128 algorithm.
  l_iv         VARCHAR2 (16) := '0123456789012345';
BEGIN
  DBMS_OUTPUT.put_line
    (   'Encrypted DATA: '
     || UTL_RAW.cast_to_varchar2
          (dbms_crypto.encrypt
                     (UTL_RAW.cast_to_raw (l_data),
                      l_algorithm,
                      UTL_RAW.cast_to_raw (l_key),
                      UTL_RAW.cast_to_raw (l_iv))));
END;
/

DECLARE
  l_plaintext       VARCHAR2 (12)
                                := 'This is data';
  l_key             VARCHAR2 (20)
                             := 'This is the Key';
  l_encrypted_text  VARCHAR2 (24);
  l_encrypted_raw   RAW (24);
  l_decrypted_text  VARCHAR2 (20);
BEGIN
  l_encrypted_raw :=
      data_crypto.encrypt_char (l_plaintext, l_key);
  l_encrypted_text :=
         data_crypto.encrypt (l_plaintext, l_key);
  l_decrypted_text :=
    data_crypto.decrypt (l_encrypted_text, l_key);
  DBMS_OUTPUT.put_line (   'Plain Text:       '
                        || l_plaintext);
  DBMS_OUTPUT.put_line (   'Key:              '
                        || l_key);
  DBMS_OUTPUT.put_line (   'Encrypted RAW:    '
                        || l_encrypted_raw);
  DBMS_OUTPUT.put_line (   'Encrypted string: '
                        || l_encrypted_text);
  DBMS_OUTPUT.put_line (   'Decrypted:        '
                        || l_decrypted_text);
END;
/

-------------------------- Dates  --------------
----------- Store Char, number, and date -----
DROP TABLE enc_val_tab;
CREATE TABLE enc_val_tab
(charval RAW(32),
dateval RAW(64),
numval  RAW(32));
EXEC data_crypto.setkey('EncryptionKey');
INSERT INTO enc_val_tab
     VALUES (data_crypto.encrypt_char (USER),
             data_crypto.encrypt_date (SYSDATE),
             data_crypto.encrypt_number (10));
COMMIT ;

COL "Number Data" format a20
COL "Character Data" format a20
COL "Date Data" format a20
-- Show stored data is encrypted. Have to convert
-- data from RAW to view.
SELECT UTL_RAW.cast_to_varchar2 (charval)
                                 "Character Data",
       UTL_RAW.cast_to_varchar2 (numval)
                                    "Number Data",
       UTL_RAW.cast_to_varchar2 (dateval)
                                      "Date Data"
  FROM enc_val_tab;
COL "Character Data" format a20
COL "Date Data" format a20
-- Show decrypted values
SELECT data_crypto.decrypt_char (charval)
                                 "Character Data",
       TO_CHAR(data_crypto.decrypt_number (numval))
                                    "Number Data",
       TO_CHAR
              (data_crypto.decrypt_date (dateval),
               'DD-MON-RRRR HH24:MI:SS')
                                      "Date Data"
  FROM enc_val_tab;

-- ** Show that current format only shows date not time **
COL date_format format a25
SELECT SYS_CONTEXT ('userenv', 'nls_date_format')
                                      DATE_FORMAT,
       SYSDATE
  FROM DUAL;
-- The current format was not set by the init.ora parameter
SHOW parameter nls_date_format

-- Create logon trigger to set date format to capture time.
-- Consider only setting format for specific users.
CREATE OR REPLACE TRIGGER nls_date_logon
  AFTER LOGON ON DATABASE
BEGIN
  IF USER IN ('SEC_MGR', 'SCOTT')
  THEN
    EXECUTE IMMEDIATE 
      'alter session set nls_date_format = ''DD-MON-RRRR HH24:MI:SS'' ';
  END IF;
END;
/
-- Show new format
CONN sec_mgr/oracle10g
SELECT SYS_CONTEXT ('userenv', 'nls_date_format')
                                      DATE_FORMAT,
       SYSDATE
  FROM DUAL;
-- NOTE the format is not set for all users
CONN system/manager
-- show new format
SELECT SYS_CONTEXT ('userenv', 'nls_date_format')
                                      DATE_FORMAT,
       SYSDATE
  FROM DUAL;
-------- show Dates now work ---------
conn sec_mgr/oracle10g
-- Remove previous records.
TRUNCATE TABLE enc_val_tab;
-- Set encryption key
EXEC data_crypto.setkey('EncryptionKey');
-- Insert values. NLS_DATE_FORMAT will preserve time.
INSERT INTO enc_val_tab
     VALUES (data_crypto.encrypt_char (USER),
             data_crypto.encrypt_date (SYSDATE),
             data_crypto.encrypt_number (10));
COMMIT ;
-- Show decrypted values
COL "Character Data" format a20
COL "Date Data" format a20
SELECT data_crypto.decrypt_char (charval)
                                 "Character Data",
       TO_CHAR(data_crypto.decrypt_number (numval))
                                    "Number Data",
       TO_CHAR
              (data_crypto.decrypt_date (dateval),
               'DD-MON-RRRR HH24:MI:SS')
                                      "Date Data"
  FROM enc_val_tab;



-------------- different keys, data, algorithms, and IVs -------------
-- Changing algorithm changes ciphertext.
COL data format a6
COL "Encrypted with Triple DES" format a34
COL "Encrypted with AES 192-bit" format a34
SELECT 'Data' DATA,
       data_crypto.encrypt_char ('Data',
                               'Key',
                               data_crypto.des3)
                     "Encrypted with Triple DES",
       data_crypto.encrypt_char ('Data',
                               'Key',
                               data_crypto.aes192)
                     "Encrypted with AES 192-bit"
  FROM DUAL;

-- changing keys
COL data format a6
COL "Encrypted with Key1" format a34
COL "Encrypted with Key2" format a34
SELECT 'Data' DATA,
       data_crypto.encrypt_char ('Data', 'Key1')
                            "Encrypted with Key1",
       data_crypto.encrypt_char ('Data', 'Key2')
                            "Encrypted with Key2"
  FROM DUAL;

-- changing data
COL "'Data1' key: 'TheSecretKey'" format a36
COL "'Data2' key: 'TheSecretKey'" format a36
SELECT data_crypto.encrypt_char ('Data1',
                               'TheSecretKey')
                    "'Data1' key: 'TheSecretKey'",
       data_crypto.encrypt_char ('Data2',
                               'TheSecretKey')
                    "'Data2' key: 'TheSecretKey'"
  FROM DUAL;
  
-- Changing IVs allows different ciphertext for
-- the same data and the same key. 
DECLARE
  -- The encryption algorithm that will be used.
  l_algorithm  PLS_INTEGER
    :=   dbms_crypto.encrypt_aes
       + dbms_crypto.chain_cbc
       + dbms_crypto.pad_pkcs5;
  -- First initialization vector. Vector has to be
  -- 16 bytes.
  l_iv1        RAW (16)
       := UTL_RAW.cast_to_raw ('0123456789012345');
  -- Second initialization vector. Vector has to be
  -- 16 bytes.
  l_iv2        RAW (16)
       := UTL_RAW.cast_to_raw ('1123456789012345');
  -- The data that will be encrypted
  l_data       VARCHAR2 (4)  := 'DATA';
  -- The key that will be used for the encryption.
  -- Key has to be 16 bytes
  l_key        VARCHAR2 (16) := 'TheEncryptionKey';
BEGIN
  DBMS_OUTPUT.put_line
    (   'Encrypted with IV1: '
     || dbms_crypto.encrypt
                     (UTL_RAW.cast_to_raw (l_data),
                      l_algorithm,
                      UTL_RAW.cast_to_raw (l_key),
                      l_iv1));
  DBMS_OUTPUT.put_line
    (   'Encrypted with IV2: '
     || dbms_crypto.encrypt
                     (UTL_RAW.cast_to_raw (l_data),
                      l_algorithm,
                      UTL_RAW.cast_to_raw (l_key),
                      l_iv2));
END;
/


-- why change is important
DROP TABLE people;
CREATE TABLE people
(ename VARCHAR2(12),
deptno VARCHAR2(16));

INSERT INTO people
  SELECT ename, deptno
    FROM scott.emp;
BREAK on deptno
SELECT   ename, deptno
    FROM people
ORDER BY deptno, ename;

EXEC data_crypto.setkey('Deptno Key');

UPDATE people
   SET deptno = data_crypto.encrypt (deptno);

COL "Encrypted Deptno" format a32
BREAK on "Encrypted Deptno"
SELECT   ename,
         UTL_RAW.cast_to_raw (deptno)
                               "Encrypted Deptno"
    FROM people
ORDER BY deptno, ename;

TRUNCATE TABLE people;

INSERT INTO people
  SELECT ename, deptno
    FROM scott.emp;

UPDATE people
   SET deptno =
               data_crypto.encrypt (deptno, ename);
SELECT   ename,
         UTL_RAW.cast_to_raw (deptno)
                               "Encrypted Deptno"
    FROM people
ORDER BY deptno, ename;


----------- Weak Keys ----------------
COL encrypted1 format a16
COL encrypted2 format a16
EXEC data_crypto.set_Algorithm(data_crypto.DES);
SELECT data_crypto.encrypt_char
                            ('Data',
                             'ThisKeyIsOver8bytes')
                                       encrypted1,
       data_crypto.encrypt_char
                        ('Data',
                         'ThisKeyIsAlsoOver8bytes')
                                       encrypted2
  FROM DUAL;

-------------------  Character Set Translations --------------
CONN sec_mgr/oracle10g@dagger
DROP TABLE people;
CREATE TABLE people
(NAME VARCHAR2(30),
salary VARCHAR2(16),
raw_salary RAW (32));
-- populate table
INSERT INTO people
            (NAME, salary)
  SELECT ename, sal
    FROM scott.emp
   WHERE ROWNUM <= 3;
-- Encrypt salaries, use name as key
-- Encrypt salary as RAW
UPDATE people
   SET raw_salary =
             data_crypto.encrypt_char (salary, NAME);
-- encrypt salary as VARCHAR2
UPDATE people
   SET salary = data_crypto.encrypt (salary, NAME);
COMMIT ;
COL decrypted_salary format a16
COL decrypted_raw_salary format a13
-- Can decrypt both RAW and varchar2
SELECT NAME,
       data_crypto.decrypt_char (raw_salary, NAME)
                             decrypted_raw_salary,
       data_crypto.decrypt (salary, NAME)
                                 decrypted_salary
  FROM people;


CONN sec_mgr/oracle10g@knox10g
CREATE DATABASE LINK dagger USING 'dagger';
-- Can still decrypt RAW
COL decrypted_raw_salary format a13
SELECT NAME,
       data_crypto.decrypt_char (raw_salary, NAME)
                             decrypted_raw_salary
  FROM people@dagger;
-- Cannot decrypt VARCHAR2
SELECT NAME,
       data_crypto.decrypt (salary, NAME)
                                 decrypted_salary
  FROM people@dagger;

-- Show character sets are different
col "Remote Character Set" format a30
col "Local Character Set" format a30
SELECT a.VALUE "Remote Character Set",
       b.VALUE "Local Character Set"
  FROM v$nls_parameters@dagger a,
       v$nls_parameters b
 WHERE a.parameter = 'NLS_CHARACTERSET'
   AND b.parameter = 'NLS_CHARACTERSET';


----------------- Data Sizes -------------
drop table enc_data_sizes;
CREATE TABLE enc_data_sizes
  (algorithm VARCHAR2(6),
  dataval VARCHAR2(256),
  enc_dataval VARCHAR2(256),
  raw_enc_dataval RAW(256));
-- Create a procedure which will be called 
-- iteratively to perform the insert operation
CREATE OR REPLACE PROCEDURE insert_enc_record (
  p_dataval  IN  VARCHAR2)
AS
BEGIN
  INSERT INTO enc_data_sizes
              (algorithm,
               dataval,
               enc_dataval,
               raw_enc_dataval)
       VALUES (data_crypto.get_algorithm,
               p_dataval,
               data_crypto.encrypt (p_dataval),
               data_crypto.encrypt_char (p_dataval));
END;
/

-- Populate table
DECLARE
  l_dataval  VARCHAR2 (32);
BEGIN
  data_crypto.setkey ('thisIsTheEncryptKey');

  FOR i IN 1 .. 32
  LOOP
    -- checking padding around block boundaries
    IF (i IN (1, 7, 8, 15, 16, 32))
    THEN
      l_dataval := RPAD ('x', i, i);
      data_crypto.set_algorithm (data_crypto.des);
      insert_enc_record (l_dataval);
      data_crypto.set_algorithm (data_crypto.des3);
      insert_enc_record (l_dataval);
      data_crypto.set_algorithm (data_crypto.aes);
      insert_enc_record (l_dataval);
      data_crypto.set_algorithm
                              (data_crypto.aes192);
      insert_enc_record (l_dataval);
      data_crypto.set_algorithm
                              (data_crypto.aes256);
      insert_enc_record (l_dataval);
      data_crypto.set_algorithm (data_crypto.rc4);
      insert_enc_record (l_dataval);
    END IF;
  END LOOP;
END;
/

-- Print results
BREAK on algorithm
COL algorithm format a9
SELECT   algorithm,
         LENGTH (dataval) "Original Size",
         LENGTH (enc_dataval) "Encrypted Size",
         LENGTH (enc_dataval) - LENGTH (dataval) "Padded Bytes"
    FROM enc_data_sizes
ORDER BY algorithm, "Original Size";
     

------------------ Transparency ----------------------
DROP TABLE people$;
-- Create table. Salary will be encrypted, so make column RAW
CREATE TABLE people$ 
AS 
  SELECT ename username, job, mgr, deptno,
     UTL_RAW.cast_from_number(sal) salary
  FROM scott.emp;

-- Create View on table
CREATE OR REPLACE VIEW people
AS
  SELECT username,
         job,
         mgr,
         deptno,
         UTL_RAW.cast_to_number (salary) salary
    FROM people$;
-- Show values
SELECT username, salary FROM people
 WHERE rownum <= 1;

-- Create package to handle encryption and decryption.
-- Security is strengthened by wrapping this package
-- and guarding execute privileges. Also, executions
-- should be audited.
CREATE OR REPLACE PACKAGE sal_crypt
AS
  FUNCTION encrypt (
    p_sal  IN  NUMBER,
    p_key  IN  VARCHAR2)
    RETURN RAW;

  FUNCTION decrypt (
    p_sal  IN  RAW,
    p_key  IN  VARCHAR2)
    RETURN NUMBER;
END;
/
CREATE OR REPLACE PACKAGE BODY sal_crypt
AS
    -- Encryption key for this example is not strong.
    -- The key will be dervied from ename. Therefore, add
    -- this enhancer to make the key more difficult to guess.
    g_key_enhancer  VARCHAR2 (16)
                            := 'd-kjhsd98qOJKD90';
  
  -- ****** Wrap this code ******
  FUNCTION encrypt (
    p_sal  IN  NUMBER,
    p_key  IN  VARCHAR2)
    RETURN RAW
  AS
  BEGIN
    -- IMPORTANT: Do all security checks prior
    -- to returning data. Check users, roles,
    -- IP address, application context values, etc.
    RETURN data_crypto.encrypt_number
                              (p_sal,
                               p_key || g_key_enhancer,
                               data_crypto.aes256);
  END;

  FUNCTION decrypt (p_sal IN RAW, p_key IN VARCHAR2)
    RETURN NUMBER
  AS
  BEGIN
    -- IMPORTANT: Do all security checks prior
    -- to returning data. Check users, roles,
    -- IP address, application context values, etc.
    RETURN data_crypto.decrypt_number
                              (p_sal,
                               p_key || g_key_enhancer,
                               data_crypto.aes256);
  END;
END;
/

-- Create table trigger
CREATE OR REPLACE TRIGGER encrypt_salary
  BEFORE INSERT OR UPDATE OF username, salary
  ON people$
  FOR EACH ROW
DECLARE
  l_old_salary  VARCHAR2 (32);
BEGIN
  IF INSERTING
  THEN
    IF (    :NEW.salary IS NOT NULL
        AND :NEW.username IS NOT NULL)
    THEN
      -- This call should be to a wrapped function which
      -- hides the key. Not done for simplicity.
      :NEW.salary :=
        sal_crypt.encrypt (utl_raw.cast_to_number(:NEW.salary),
                           :NEW.username);
    END IF;
  ELSE                         -- must be updating
    BEGIN
      -- if salary is not encrypted already, this will error
      l_old_salary :=
        sal_crypt.decrypt (:OLD.salary,
                           :OLD.username);
    EXCEPTION
      WHEN OTHERS
      THEN
        l_old_salary := :OLD.salary;
    END;

    :NEW.salary :=
       sal_crypt.encrypt  (utl_raw.cast_to_number(l_old_salary),
                          :NEW.username);
  END IF;
END;
/

-- Encrypt data
UPDATE people$
   SET salary = salary;
COMMIT ;
-- View no longer works as data is not decrypted
SELECT username, salary FROM people
 WHERE rownum <= 1;
-- Update view to decrypt data
CREATE OR REPLACE VIEW people
AS
  SELECT username,
         job,
         mgr,
         deptno,
         sal_crypt.decrypt(salary,username) salary
    FROM people$;
-- Show we can decrypt
COL "Decrypted from View" format a20
COL "Stored Value" format a64
SELECT a.username,
       a.salary "Decrypted from View",
       UTL_RAW.cast_to_varchar2 (b.salary)
                                   "Stored Value"
  FROM people a, people$ b
 WHERE a.username = b.username AND ROWNUM <= 2;
 
 ---------------- LOBs --------------------
 CREATE TABLE docs
  (doc_id NUMBER(4),
  doc     CLOB);
 
 INSERT INTO docs
      VALUES (1, 'This is CLOB data');
 COMMIT ;
 
 EXEC data_crypto.setkey('This is the Key');
 UPDATE docs
    SET doc = data_crypto.encrypt (doc);
 COL Encrypted format a32
 COL Decrypted format a32
 SELECT doc encrypted,
        data_crypto.decrypt (doc) decrypted
   FROM docs;

CONN system/manager
CREATE OR REPLACE DIRECTORY doc_dir AS
'C:\tmp\esbd';

GRANT ALL ON DIRECTORY doc_dir TO sec_mgr;
CONN sec_mgr/oracle10g
DROP TABLE docs;
CREATE TABLE docs
  (doc_id NUMBER(4),
  doc     BLOB)
/

DECLARE
  l_blob   BLOB;
  l_bfile  BFILE;
BEGIN
  INSERT INTO docs
       VALUES (1, EMPTY_BLOB ())
    RETURNING doc
         INTO l_blob;
  l_bfile := BFILENAME ('DOC_DIR', 'sample.xml');
  DBMS_LOB.fileopen (l_bfile);
  DBMS_LOB.loadfromfile
                     (l_blob,
                      l_bfile,
                      DBMS_LOB.getlength (l_bfile));
  DBMS_LOB.fileclose (l_bfile);
END;
/

EXEC data_crypto.setkey('This is the Key');
UPDATE docs
   SET doc = data_crypto.encrypt (doc);
COMMIT ;

DECLARE
  l_encrypted_blob  BLOB;
  l_decrypted_blob  BLOB;
  l_decrypted_clob  CLOB;
  l_lang            NUMBER := DBMS_LOB.default_lang_ctx;
  l_warning         NUMBER;
  l_t_offset        NUMBER := 1;
  l_src_offset      NUMBER := 1;
BEGIN
  SELECT doc
    INTO l_encrypted_blob
    FROM docs;

  DBMS_LOB.createtemporary (l_decrypted_blob, TRUE);
  DBMS_LOB.createtemporary (l_decrypted_clob, TRUE);
  -- decrypt BLOB
  l_decrypted_blob := data_crypto.decrypt (l_encrypted_blob);
  -- convert to CLOB for display
  DBMS_LOB.converttoclob (l_decrypted_clob,
                          l_decrypted_blob,
                          DBMS_LOB.lobmaxsize,
                          l_t_offset,
                          l_src_offset,
                          DBMS_LOB.default_csid,
                          l_lang,
                          l_warning
                         );
  DBMS_OUTPUT.put_line (l_decrypted_clob);
END;
/

-------------- Hashing ---------
COL "Hashed Data" format a30
COL  "Encrypted Data" format a30
SELECT data_crypto.hash ('Data')
                                    "Hashed Data",
       data_crypto.encrypt ('Data', 'k')
                                 "Encrypted Data"
  FROM DUAL;

-- password table
DROP TABLE people;
-- Create table with passwords
CREATE TABLE people
(ename VARCHAR2(12),
PASSWORD VARCHAR2(30));
INSERT INTO people
  SELECT ename, INITCAP (ename)
    FROM scott.emp;
-- Hash passwords
UPDATE people
   SET PASSWORD = data_crypto.HASH (PASSWORD);
COMMIT ;
-- show passwords are now protected
COL password format a40
SELECT ename,
       UTL_RAW.cast_to_raw (PASSWORD) PASSWORD
  FROM people
 WHERE ROWNUM <= 3;

-- Password Authentication function
CREATE OR REPLACE FUNCTION is_user_auth (
  p_username  IN  VARCHAR2,
  p_password  IN  VARCHAR2
)
  RETURN BOOLEAN
AS
  l_hashed_password  VARCHAR2 (20);
  l_user_is_authed   BOOLEAN       := FALSE;
BEGIN
  FOR rec IN (SELECT PASSWORD
                FROM people
               WHERE UPPER (ename) = UPPER (p_username))
  LOOP
    l_hashed_password := data_crypto.HASH (p_password);

    IF l_hashed_password = rec.PASSWORD
    THEN
      l_user_is_authed := TRUE;
    END IF;
  END LOOP;

  RETURN l_user_is_authed;
END;
/

BEGIN
  IF (is_user_auth ('scott', 'scott'))
  THEN
    DBMS_OUTPUT.put_line ('Scott authenticated with "scott"');
  ELSE
    DBMS_OUTPUT.put_line ('Scott could not authenticate with "scott"');
  END IF;

  IF (is_user_auth ('scott', 'Scott'))
  THEN
    DBMS_OUTPUT.put_line ('Scott authenticated with "Scott"');
  ELSE
    DBMS_OUTPUT.put_line ('Scott could not authenticate with "Scott"');
  END IF;
END;
/

-------------------------- MAC ----------------
-- reset passwords
COL password format a40
UPDATE people
   SET PASSWORD = INITCAP (ename);
-- compute HMAC using key
UPDATE people
   SET PASSWORD =
               data_crypto.mac (PASSWORD, 'MAC Password Key');
SELECT ename, UTL_RAW.cast_to_raw (PASSWORD) PASSWORD
  FROM people
 WHERE ROWNUM <= 3;

CREATE OR REPLACE FUNCTION is_user_auth (
  p_username  IN  VARCHAR2,
  p_password  IN  VARCHAR2
)
  RETURN BOOLEAN
AS
  l_hashed_password  VARCHAR2 (20);
  l_user_is_authed   BOOLEAN       := FALSE;
BEGIN
  FOR rec IN (SELECT PASSWORD
                FROM people
               WHERE UPPER (ename) = UPPER (p_username))
  LOOP
    l_hashed_password := data_crypto.HASH (p_password);

    IF l_hashed_password = rec.PASSWORD
    THEN
      l_user_is_authed := TRUE;
    END IF;
  END LOOP;

  RETURN l_user_is_authed;
END;
/

BEGIN
  IF (is_user_auth ('scott', 'Scott'))
  THEN
    DBMS_OUTPUT.put_line ('Scott authenticated with "Scott"');
  ELSE
    DBMS_OUTPUT.put_line ('Scott could not authenticate with "Scott"');
  END IF;
END;
/

CREATE OR REPLACE FUNCTION is_user_auth (
  p_username  IN  VARCHAR2,
  p_password  IN  VARCHAR2
)
  RETURN BOOLEAN
AS
  l_hashed_password  VARCHAR2 (20);
  l_user_is_authed   BOOLEAN       := FALSE;
BEGIN
  FOR rec IN (SELECT PASSWORD
                FROM people
               WHERE UPPER (ename) = UPPER (p_username))
  LOOP
    l_hashed_password := data_crypto.mac (p_password, 
                                          'MAC Password Key');

    IF l_hashed_password = rec.PASSWORD
    THEN
      l_user_is_authed := TRUE;
    END IF;
  END LOOP;

  RETURN l_user_is_authed;
END;
/
BEGIN
  IF (is_user_auth ('scott', 'Scott'))
  THEN
    DBMS_OUTPUT.put_line ('Scott authenticated with "Scott"');
  ELSE
    DBMS_OUTPUT.put_line ('Scott could not authenticate with "Scott"');
  END IF;
END;
/


----------------------------- PERFORMANCE -------------------
-- Capture wall clock
SET timing on
-- Time Basic Loop
DECLARE
  l_data        DATE;
  l_start_time  NUMBER;
  l_end_time    NUMBER;
BEGIN
  l_start_time := DBMS_UTILITY.get_time;

  FOR i IN 1 .. 100000
  LOOP
    SELECT SYSDATE
      INTO l_data
      FROM DUAL;
  END LOOP;

  l_end_time := DBMS_UTILITY.get_time;
  DBMS_OUTPUT.put_line (   'Loop Time: '
                        || (l_end_time - l_start_time) / 100
                        || ' seconds');
END;
/

DECLARE
  l_algo        PLS_INTEGER
    :=   dbms_crypto.encrypt_aes
       + dbms_crypto.chain_cbc
       + dbms_crypto.pad_pkcs5;
  l_key         RAW (16)
                 := UTL_RAW.cast_to_raw ('0123456789012345');
  l_data        RAW (20)
             := UTL_RAW.cast_to_raw ('01234567890123456789');
  l_enc_data    RAW (40);
  l_start_time  NUMBER;
  l_end_time    NUMBER;
BEGIN
  l_start_time := DBMS_UTILITY.get_time;

  FOR i IN 1 .. 100000
  LOOP
    SELECT dbms_crypto.encrypt (l_data, l_algo, l_key)
      INTO l_enc_data
      FROM DUAL;
  END LOOP;

  l_end_time := DBMS_UTILITY.get_time;
  DBMS_OUTPUT.put_line (   'AES Encrypt');
  DBMS_OUTPUT.put_line (   'Time: '
                        || (l_end_time - l_start_time) / 100
                        || ' seconds');
END;
/

----------------------- Key Management ------------
CREATE OR REPLACE FUNCTION enc_some_data (
  p_data  IN  VARCHAR2 DEFAULT NULL,
  p_key    IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
  l_key        RAW (64);
  l_other_val  RAW (64);
BEGIN
  l_other_val :=
    UTL_RAW.cast_to_raw (   SYS_CONTEXT ('userenv',
                                         'instance_name')
                         || SYS_CONTEXT ('userenv',
                                         'current_schema'));
  l_key :=
    UTL_RAW.bit_xor (UTL_RAW.cast_to_raw (p_key),
                    l_other_val);
  RETURN data_crypto.encrypt
                            (p_data,
                             UTL_RAW.cast_to_varchar2 (l_key));
END;
/

CREATE OR REPLACE FUNCTION dec_some_data (
  p_data  IN  VARCHAR2 DEFAULT NULL,
  p_key    IN  VARCHAR2 DEFAULT NULL)
  RETURN VARCHAR2
AS
  l_key        RAW (64);
  l_other_val  RAW (64);
BEGIN
  l_other_val :=
    UTL_RAW.cast_to_raw (   SYS_CONTEXT ('userenv',
                                         'instance_name')
                         || SYS_CONTEXT ('userenv',
                                         'current_schema'));
  l_key :=
    UTL_RAW.bit_xor (UTL_RAW.cast_to_raw (p_key),
                    l_other_val);
  RETURN data_crypto.decrypt
                            (p_data,
                             UTL_RAW.cast_to_varchar2 (l_key));
END;
/

-- Wrap and load
SELECT   text
    FROM user_source
   WHERE NAME = 'ENC_SOME_DATA'
ORDER BY line;
spool off
edit ch13.log
