CREATE OR REPLACE PROCEDURE deactivate_user (
  p_username  IN  VARCHAR2)
AS
  TYPE l_role_list_type IS TABLE OF VARCHAR2 (30)
    INDEX BY BINARY_INTEGER;

  l_role_list    l_role_list_type;
  l_role_index   BINARY_INTEGER   := 1;
  l_role_string  VARCHAR2 (32767);
  l_username     VARCHAR2 (30)    := upper(p_username);
BEGIN
  -- revoke System priviliges granted directly to the user
  FOR rec IN (SELECT PRIVILEGE, admin_option
                FROM dba_sys_privs
               WHERE grantee = l_username)
  LOOP
    IF (rec.admin_option = 'NO')
    THEN
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.PRIVILEGE
                            || ' to '
                            || l_username
                            || ';');
    ELSE
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.PRIVILEGE
                            || ' to '
                            || l_username
                            || ' WITH ADMIN OPTION;');
    END IF;

    EXECUTE IMMEDIATE    'REVOKE '
                      || rec.PRIVILEGE
                      || ' FROM '
                      || l_username;
  END LOOP;

  -- revoke Object priviliges granted directly to the user
  FOR rec IN (SELECT owner,
                     table_name,
                     PRIVILEGE,
                     grantable
                FROM dba_tab_privs
               WHERE grantee = l_username)
  LOOP
    IF (rec.grantable = 'NO')
    THEN
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.PRIVILEGE
                            || ' ON '
                            || rec.owner
                            || '.'
                            || rec.table_name
                            || ' to '
                            || l_username
                            || ';');
    ELSE
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.PRIVILEGE
                            || ' ON '
                            || rec.owner
                            || '.'
                            || rec.table_name
                            || ' to '
                            || l_username
                            || ' WITH ADMIN OPTION;');
    END IF;

    EXECUTE IMMEDIATE    'REVOKE '
                      || rec.PRIVILEGE
                      || ' ON '
                      || rec.owner
                      || '.'
                      || rec.table_name
                      || ' FROM '
                      || l_username;
  END LOOP;

  -- revoke roles granted directly to the user
  FOR rec IN (SELECT *
                FROM dba_role_privs
               WHERE grantee = l_username)
  LOOP
    IF (rec.admin_option = 'NO')
    THEN
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.granted_role
                            || ' to '
                            || l_username
                            || ';');
    ELSE
      DBMS_OUTPUT.put_line (   'grant '
                            || rec.granted_role
                            || ' to '
                            || l_username
                            || ' WITH ADMIN OPTION;');
    END IF;

    IF (rec.default_role = 'YES')
    THEN
      l_role_list (l_role_index) := rec.granted_role;
      l_role_index := l_role_index + 1;
    END IF;

    EXECUTE IMMEDIATE    'REVOKE '
                      || rec.granted_role
                      || ' FROM '
                      || l_username;
  END LOOP;

  IF l_role_index > 1
  THEN
    l_role_string :=
         'alter user ' || l_username ||' default roles '
      || l_role_list (1);

    FOR i IN 2 .. l_role_index - 1
    LOOP
      l_role_string :=
             l_role_string || ', ' || l_role_list (i);
    END LOOP;

    DBMS_OUTPUT.put_line (l_role_string || ';');
  END IF;
END;
/


---- TEST --------
SET serveroutput on
SPOOL scottPrivs.sql
EXEC deactivate_user('scott')
SPOOL off
@scottPrivs.sql
