CREATE TABLE passwords
  (
  username VARCHAR2(30),
  passwd   VARCHAR2(30),
  verifier VARCHAR2(30)
  )
/
ALTER TABLE PASSWORDS ADD (
  CONSTRAINT PWD_PK PRIMARY KEY (PASSWD, USERNAME));

CREATE INDEX VERIFIER ON PASSWORDS
(USERNAME, VERIFIER);

CREATE OR REPLACE PROCEDURE populate_passwords_tab (
  p_password  IN  VARCHAR2 DEFAULT NULL)
AS
  l_new_password_verifier  dba_users.PASSWORD%TYPE;
  l_password               dba_users.PASSWORD%TYPE
                            := UPPER (p_password);
BEGIN
  FOR rec IN (SELECT username, password
                FROM dba_users)
  LOOP
    IF (p_password IS NULL)
    THEN
      -- password is either passed as parameter 
      -- or set to user's name
      l_password := rec.username;
    END IF;

    -- create new password verifier
    EXECUTE IMMEDIATE    'alter user '
                      || rec.username
                      || ' identified by '
                      || l_password;
    -- retrieve new verifier
    SELECT password
      INTO l_new_password_verifier
      FROM dba_users
     WHERE username = rec.username;
    -- insert value into passwords table
    INSERT INTO passwords
         VALUES (rec.username,
                 l_password,
                 l_new_password_verifier);
    -- set password back to its original value
    EXECUTE IMMEDIATE    'alter user '
                      || rec.username
                      || ' identified by values '''
                      || rec.password
                      || '''';
  END LOOP;
END;
/

-- Populate table
SET timing off
TRUNCATE TABLE passwords;
-- sets all passwords to that of user's name
EXEC populate_passwords_tab
-- check for manager, a common password for DBA accounts
EXEC populate_passwords_tab ('manager')
-- check for your company's name here
EXEC populate_passwords_tab ('oracle')
-- SCOTT's default password
EXEC populate_passwords_tab ('tiger')
-- SYS' default password
EXEC populate_passwords_tab ('change_on_install')
-- common password people use
EXEC populate_passwords_tab ('password')
COMMIT ;

-- Check passwords
CREATE OR REPLACE PROCEDURE check_passwords
AS
BEGIN
  FOR rec IN (SELECT username,
                     PASSWORD,
                     account_status
                FROM dba_users)
  LOOP
    FOR irec IN (SELECT *
                   FROM passwords
                  WHERE username = rec.username
                    AND verifier = rec.PASSWORD)
    LOOP
      DBMS_OUTPUT.put_line
                ('------------------------------');
      DBMS_OUTPUT.put_line (   'Password for '
                            || rec.username
                            || ' is '
                            || irec.passwd);
      DBMS_OUTPUT.put_line
                          (   'Account Status is '
                           || rec.account_status);
    END LOOP;
  END LOOP;
END;
/

SET serveroutput on
SET timing on
EXEC check_passwords
