CONN sys as sysdba

CREATE OR REPLACE FUNCTION is_password_strong (
  p_username      VARCHAR2,
  p_new_password  VARCHAR2,
  p_old_password  VARCHAR2)
  -- return TRUE if password is strong enough
RETURN BOOLEAN
AS
  l_return_val  BOOLEAN := TRUE;
BEGIN
  -- Check to be sure password is not the same as username
  IF UPPER (p_new_password) = UPPER (p_username)
  THEN
    l_return_val := FALSE;
    raise_application_error
                    (-20001,
                     'Password same as user name');
  END IF;

  -- force user to change password to something new
  IF UPPER (p_new_password) =
                            UPPER (p_old_password)
  THEN
    l_return_val := FALSE;
    raise_application_error
      (-20004,
       'Password has to be different than old password');
  END IF;

  -- Check for list of predictable passwords
  IF LOWER (p_new_password) IN
       ('manager',
        'change_on_install',
        'oracle',
        'password')
  THEN
    l_return_val := FALSE;
    raise_application_error
                   (-20002,
                    'Password is too predictable');
  END IF;

  -- make sure password contains at least one digit
  IF (regexp_like (p_new_password, '[0123456789]') =
                                             FALSE)
  THEN
    l_return_val := FALSE;
    raise_application_error
             (-20003,
              'Password needs at least one digit');
  END IF;

  -- make sure password is at least six characters
  IF LENGTH (p_new_password) <= 6
  THEN
    l_return_val := FALSE;
    raise_application_error
                         (-20005,
                          'Password is too short');
  END IF;

  RETURN l_return_val;
END;
/

CONN sec_mgr
ALTER USER scott PROFILE DEFAULT;
DROP PROFILE strong_pwd cascade;
CREATE PROFILE strong_pwd LIMIT
  PASSWORD_LIFE_TIME 90
  PASSWORD_GRACE_TIME 15
  PASSWORD_REUSE_TIME 180
  PASSWORD_REUSE_MAX UNLIMITED
  FAILED_LOGIN_ATTEMPTS 5
  PASSWORD_LOCK_TIME .5
  PASSWORD_VERIFY_FUNCTION is_password_strong;

SELECT resource_name, LIMIT
  FROM dba_profiles
 WHERE resource_type = 'PASSWORD'
   AND PROFILE = 'STRONG_PWD';

-- assign to scott
ALTER USER scott PROFILE strong_pwd;

-- reset scott's password
ALTER USER scott IDENTIFIED BY scott;
ALTER USER scott IDENTIFIED BY manager;
ALTER USER scott IDENTIFIED BY nodigit;
-- since this is not SCOTT, old password is null
ALTER USER scott IDENTIFIED BY tiger;
ALTER USER scott IDENTIFIED BY short1;

-- force a change
ALTER  USER scott PASSWORD EXPIRE;

conn scott/tiger

ALTER USER scott PROFILE DEFAULT;
