CONN sec_mgr/oracle10g
-- reset
drop synonym app_public.enable_person_view;
drop SYNONYM app_public.person_view;
drop synonym app_public.reset_state;
drop role person_view_app_role;

-- create synonym for View
CREATE SYNONYM app_public.person_view 
           FOR sec_mgr.person_view;

-- create a password protected role
CREATE ROLE person_view_app_role
  IDENTIFIED BY thisistherolepassword;
-- grant privileges to select on view to role
GRANT SELECT ON person_view TO person_view_app_role;
-- grant role to connection pool (application) schema
GRANT person_view_app_role TO app_public;
-- disable role forcing applicatoin to set with password
ALTER USER app_public DEFAULT ROLE NONE;

CREATE OR REPLACE PROCEDURE enable_person_view (
  p_user      IN  VARCHAR2,
  p_password  IN  VARCHAR2)
AUTHID CURRENT_USER
AS
BEGIN
  -- Set Client Identifier. Value can be audited.
  client_info_mgr.set_info (p_user);
  -- Set PL/SQL Package info for user.
  -- This value can be used for fine grained access control
  user_info.set_name (p_user);
  -- Enable role which allows access to View
  DBMS_SESSION.set_role
        (   'person_view_app_role identified by '
         || p_password);
END;
/
-- create synonym for application
CREATE SYNONYM app_public.enable_person_view 
           FOR sec_mgr.enable_person_view;
-- grant execute privileges to application schema
GRANT EXECUTE ON enable_person_view TO app_public;


CREATE OR REPLACE PROCEDURE reset_state
AUTHID CURRENT_USER
AS
BEGIN
  -- Clear the Client Identifier.
  DBMS_SESSION.clear_identifier;
  -- clear the PL/SQL package state
  DBMS_SESSION.reset_package;
  -- Disable roles
  DBMS_SESSION.set_role ('none');
END;
/
-- create synonym for application
CREATE SYNONYM app_public.reset_state 
           FOR sec_mgr.reset_state;
-- grant execute privileges to application role
GRANT EXECUTE ON reset_state TO app_public;
