CONN sys/oracle as sysdba
GRANT SELECT ON sysauth$ TO sec_mgr WITH GRANT OPTION;
GRANT SELECT ON user$ TO sec_mgr WITH GRANT OPTION;
GRANT SELECT ON dba_tab_privs TO sec_mgr WITH GRANT OPTION;
GRANT SELECT ON dba_sys_privs TO sec_mgr WITH GRANT OPTION;


CONN sec_mgr/oracle10g
DROP PUBLIC SYNONYM user_object_privs;
DROP PUBLIC SYNONYM user_system_privs;
DROP PUBLIC SYNONYM all_user_roles;

CREATE OR REPLACE VIEW all_user_roles
AS
  SELECT DISTINCT usr.NAME granted_role
             FROM (SELECT     *
                         FROM SYS.sysauth$
                   CONNECT BY PRIOR privilege# = grantee#
                   START WITH grantee# = UID
                           OR grantee# = 1) sauth,
                  SYS.user$ usr
            WHERE usr.user# = sauth.privilege#
  UNION ALL
  SELECT USER
    FROM DUAL
  UNION ALL
  SELECT 'PUBLIC'
    FROM DUAL
/
GRANT SELECT ON all_user_roles TO PUBLIC;
CREATE PUBLIC SYNONYM all_user_roles FOR all_user_roles;
SELECT *
  FROM all_user_roles;

CREATE OR REPLACE VIEW user_system_privs
AS
  SELECT DISTINCT PRIVILEGE
             FROM dba_sys_privs
            WHERE grantee IN (SELECT *
                                FROM all_user_roles)
/
GRANT SELECT ON user_system_privs TO PUBLIC;
CREATE PUBLIC SYNONYM user_system_privs 
  FOR user_system_privs;
SELECT *
  FROM user_system_privs;

CREATE OR REPLACE VIEW user_object_privs
AS
  SELECT DISTINCT PRIVILEGE, owner, table_name
             FROM dba_tab_privs
            WHERE grantee IN (SELECT *
                                FROM all_user_roles)
/
GRANT SELECT ON user_object_privs TO PUBLIC;
CREATE PUBLIC SYNONYM user_object_privs 
  FOR user_object_privs;
SELECT COUNT (*)
  FROM user_object_privs;
SELECT PRIVILEGE, table_name
  FROM user_object_privs
 WHERE owner = 'SCOTT';


CONN system/manager
SELECT COUNT (*)
  FROM (SELECT DISTINCT PRIVILEGE
                   FROM dba_sys_privs);
SELECT COUNT (*)
  FROM user_system_privs;


CREATE OR REPLACE FUNCTION get_uid (p_username IN VARCHAR2)
  RETURN NUMBER
AS
  l_uid  NUMBER;
BEGIN
  SELECT user#
    INTO l_uid
    FROM SYS.user$
   WHERE NAME = UPPER (p_username);
  RETURN l_uid;
END;
/
CREATE OR REPLACE FUNCTION show_all_roles (
  p_username  IN  VARCHAR2 DEFAULT USER)
  RETURN VARCHAR2
AS
  l_roles  VARCHAR2 (32767);
BEGIN
  FOR rec IN
    (SELECT DISTINCT u2.NAME granted_role
                FROM (SELECT     *
                            FROM SYS.sysauth$
                      CONNECT BY PRIOR privilege# =
                                              grantee#
                      START WITH grantee# =
                                   get_uid (p_username)
                              OR grantee# = 1) sa,
                     SYS.user$ u2
               WHERE u2.user# = sa.privilege#
     UNION ALL
     SELECT UPPER (p_username)
       FROM DUAL
     UNION ALL
     SELECT 'PUBLIC'
       FROM DUAL)
  LOOP
    l_roles :=
      l_roles || '''' || rec.granted_role || ''''
      || ',';
  END LOOP;

  l_roles := SUBSTR (l_roles, 1, LENGTH (l_roles) - 1);
  RETURN l_roles;
END;
/
SELECT show_all_roles
  FROM DUAL;

SELECT show_all_roles ('scott')
  FROM DUAL;

SELECT DISTINCT PRIVILEGE,  owner, table_name
           FROM dba_tab_privs
          WHERE grantee IN (show_all_roles)
/
-- grant select on dba_tab_privs to sec_mgr;

CREATE OR REPLACE PROCEDURE show_obj_privs (
  p_username  IN  VARCHAR2 DEFAULT USER)
AS
  l_roles  VARCHAR2 (32767)
                          := show_all_roles (p_username);
  l_sql    VARCHAR2 (32767);
  l_privs  VARCHAR2 (32767);
BEGIN
-- object privs
  l_sql :=
       'SELECT DISTINCT PRIVILEGE '
    || 'FROM dba_tab_privs '
    || 'WHERE grantee in ('
    || l_roles
    || ')';
  EXECUTE IMMEDIATE l_sql
               INTO l_privs;
  DBMS_OUTPUT.put_line (l_sql);
END;
/

exec show_obj_privs

SELECT DISTINCT PRIVILEGE, table_name
           FROM dba_tab_privs
          WHERE grantee IN (SELECT show_all_roles
                              FROM DUAL)
/
SELECT DISTINCT PRIVILEGE, owner, table_name
           FROM dba_tab_privs
          WHERE grantee IN (SELECT show_all_roles
                              FROM DUAL)
/
SELECT DISTINCT PRIVILEGE, owner, table_name
           FROM dba_tab_privs
          WHERE grantee IN (
                          SELECT show_all_roles ('scott')
                            FROM DUAL)
/
SELECT DISTINCT PRIVILEGE
           FROM dba_sys_privs
          WHERE grantee IN (SELECT *
                              FROM role_hierarchy)
/
