conn scott/tiger
drop table aud_emp;
drop table emp_copy;

spool ch8.out
create table emp_copy as select * from emp;

CREATE TABLE aud_emp (
  USERNAME    varchar2(30) 
    default sys_context('userenv','session_user'),
  ACTION      varchar2(6),
  EMPNO       number(4),
  COLUMN_NAME varchar2(255),
  CLIENT_ID   varchar2(255) 
    default sys_context('userenv','client_identifier'),
  OLD_VALUE   varchar2(10),
  NEW_VALUE   varchar2(10),
  SQL_TEXT    varchar2(4000),
  ACTION_DATE date default sysdate
)
/

CREATE OR REPLACE 
PROCEDURE audit_emp (p_username    in varchar2,
                     p_action      in varchar2,
                     p_empno       in number,
                     p_column_name in varchar2,
                     p_client_id   in varchar2,
                     p_old_value   in varchar2,
                     p_new_value   in varchar2,
                     p_sql         in varchar2,
                     p_action_date in date)
AS
BEGIN
-- check data format and length
-- not shown here
  insert into aud_emp 
    (username, action, empno, column_name, 
       client_id, old_value, new_value, sql_text, action_date)
  values
    (p_username, p_action, p_empno, p_column_name, 
       p_client_id, p_old_value, p_new_value, p_sql, p_action_date);
END;
/

conn scott/tiger
set serveroutput on format wrapped
CREATE OR REPLACE 
PROCEDURE show_aud_emp
AS
BEGIN
  FOR rec in (select * from aud_emp order by action_date desc)
  LOOP
    dbms_output.put_line('User:      ' || rec.username);
    dbms_output.put_line('Action:    ' || rec.action);
    dbms_output.put_line('Empno:     ' || rec.empno);
    dbms_output.put_line('Client ID: ' || rec.client_id);
    dbms_output.put_line('Column:    ' || rec.column_name);
    dbms_output.put_line('Old Value: ' || rec.old_value);
    dbms_output.put_line('New Value: ' || rec.new_value);
    dbms_output.put_line('SQL Text : ' || rec.sql_text);
    dbms_output.put_line('Date:      ' || 
      to_char(rec.action_date,'Mon-DD-YY HH24:MI'));
    dbms_output.put_line('------------------------------');
  END LOOP;
END;
/
exec show_aud_emp


CREATE OR REPLACE TRIGGER update_emp_sal_trig
BEFORE UPDATE OF SAL
ON EMP_COPY
FOR EACH ROW
DECLARE
  l_sql_text ora_name_list_t;
  l_stmt CLOB;
n number;
BEGIN

  n := ora_sql_txt(l_sql_text);

  FOR i IN 1..n LOOP
   l_stmt := l_stmt || l_sql_text(i);
  END LOOP;

  audit_emp (
    p_username    => USER,
    p_action      => 'UPDATE',
    p_empno       => :old.empno,
    p_client_id   => sys_context('userenv','client_identifier'),
    p_column_name => 'SAL',
    p_old_value   => to_char(:old.sal),
    p_new_value   => to_char(:new.sal),
    p_sql         => nvl(l_stmt,'no sql'),
    p_action_date => sysdate);
END;
/
sho errors

update scott.emp_copy
  set sal = sal * 1.1
  where deptno = 10;
commit;

exec show_aud_emp;

spool off
drop table aud_emp;
drop table emp_copy;
edit ch8.out
