SPOOL audit.log
CONNECT system/manager
-- DB has to be setup for auditing
-- change by "alter system set audit_trail='DB' scope=spfile"
-- audit connect

BEGIN
  FOR rec IN
    (SELECT username,
            action_name,
            TO_CHAR (TIMESTAMP, 'Mon-DD HH24:MI')
                                            LOGON,
            TO_CHAR (logoff_time,
                     'Mon-DD HH24:MI') LOGOFF,
            priv_used,
            comment_text
       FROM dba_audit_trail)
  LOOP
    DBMS_OUTPUT.put_line (   'User:      '
                          || rec.username);
    DBMS_OUTPUT.put_line (   'Action:    '
                          || rec.action_name);
    DBMS_OUTPUT.put_line (   'Logon:     '
                          || rec.LOGON);
    DBMS_OUTPUT.put_line (   'Logoff:    '
                          || rec.LOGOFF);
    DBMS_OUTPUT.put_line (   'Priv Used: '
                          || rec.priv_used);
    DBMS_OUTPUT.put_line (   'Comments:  '
                          || rec.comment_text);
    DBMS_OUTPUT.put_line
      ('--------------- End of Record ---------------');
  END LOOP;
END;
/

CONN sec_mgr/oracle10g
-- DB has to be setup for auditing
-- change by "alter system set audit_trail='DB' scope=spfile"
SHOW parameter audit_trail
drop table t;
CREATE TABLE t AS SELECT * FROM DUAL;
-- audit selects on T for failures
AUDIT SELECT ON t
  BY ACCESS WHENEVER NOT SUCCESSFUL;

CONN scott/tiger
SELECT *
  FROM sec_mgr.t;

conn sec_mgr/oracle10g
BEGIN
  FOR rec IN (SELECT audit_type,
                     db_user,
                     object_schema,
                     object_name,
                     extended_timestamp,
                     sql_text
                FROM dba_common_audit_trail)
  LOOP
    DBMS_OUTPUT.put_line (   'Audit Type: '
                          || rec.audit_type);
    DBMS_OUTPUT.put_line (   'Who:        '
                          || rec.db_user);
    DBMS_OUTPUT.put_line (   'What:       '
                          || rec.object_schema
                          || '.'
                          || rec.object_name);
    DBMS_OUTPUT.put_line (   'When:       '
                          || rec.extended_timestamp);
    DBMS_OUTPUT.put_line (   'How:        '
                          || rec.sql_text);
    DBMS_OUTPUT.put_line
      ('--------------- End of Record ---------------');
  END LOOP;
END;
/

NOAUDIT SELECT ON scott.emp;
-- nothing in audit trail
DELETE FROM SYS.aud$;
COMMIT ;
SELECT COUNT (*)
  FROM SYS.dba_audit_trail;

conn sec_mgr/oracle10g
CREATE OR REPLACE TRIGGER set_default_client_info
  AFTER LOGON ON DATABASE
DECLARE
  l_module  v$session.module%TYPE;
BEGIN
  SELECT UPPER (module)
    INTO l_module
    FROM v$process a, v$session b
   WHERE a.addr = b.paddr
     AND b.audsid = USERENV ('sessionid');
  DBMS_SESSION.set_identifier
                      (   SYS_CONTEXT
                                     ('userenv',
                                      'ip_address')
                       || ':'
                       || l_module);
END;
/
AUDIT SELECT ON scott.emp BY ACCESS;

CONN system/manager
SELECT ename, sal
  FROM scott.emp
 WHERE sal < (SELECT sal
                FROM scott.emp
               WHERE ename = 'SCOTT')
   AND deptno = (SELECT deptno
                   FROM scott.emp
                  WHERE ename = 'SCOTT');


-- grant privs to demo to select on scott.emp
CONN scott/tiger

-- set client identifier, this shows up in audit trail
--exec dbms_session.set_identifier('Application APP_A');
--exec DBMS_APPLICATION_INFO.SET_CLIENT_INFO (sys_context('userenv','ip_address'));

-- execute two very different queries
SELECT SUM (sal) FROM scott.emp;
SELECT ename FROM scott.emp
 WHERE 1 = 2;

CONN sec_mgr/oracle10g
SELECT COUNT (*)
  FROM dba_common_audit_trail;

CREATE OR REPLACE PROCEDURE show_aud
AS
BEGIN
  FOR rec IN (SELECT   db_user,
                       client_id,
                       object_schema,
                       object_name,
                       extended_timestamp,
                       sql_text,
                       statementid
                  FROM dba_common_audit_trail
              GROUP BY db_user,
                       statementid,
                       sql_text,
                       object_schema,
                       object_name,
                       client_id,
                       extended_timestamp
              ORDER BY extended_timestamp ASC)
  LOOP
    DBMS_OUTPUT.put_line ('Who:  ' || rec.db_user);
    DBMS_OUTPUT.put_line (   'What: '
                          || rec.object_schema
                          || '.'
                          || rec.object_name);
    DBMS_OUTPUT.put_line (   'Where: '
                          || rec.client_id);
    DBMS_OUTPUT.put_line
                 (   'When:  '
                  || TO_CHAR
                          (rec.extended_timestamp,
                           'Mon-DD HH24:MI'));
    DBMS_OUTPUT.put_line ('How:   '
                          || rec.sql_text);
    DBMS_OUTPUT.put_line
      ('-------------- End of Record --------------');
  END LOOP;
END;
/

EXEC show_aud


--------- timing ---------------
SET timing on

BEGIN
  FOR rec IN 1 .. 100000
  LOOP
    FOR irec IN (SELECT ename
                   FROM scott.emp)
    LOOP
      NULL;
    END LOOP;
  END LOOP;
END;
/

-- connect as a user without access


DELETE FROM SYS.aud$;
NOAUDIT SELECT ON scott.emp;
COMMIT ;
REVOKE SELECT ON scott.emp FROM demo;


EDIT audit.log
EXIT


CONN system/manager
-- DB has to be setup for auditing
-- change by "alter system set audit_trail='DB' scope=spfile"
SHOW parameter audit_trail


-- audit for selects on emp everytime for success and failure
AUDIT SELECT ON scott.emp BY ACCESS WHENEVER NOT SUCCESSFUL;

BEGIN
  FOR rec IN (SELECT audit_type,
                     db_user,
                     object_schema,
                     object_name,
                     extended_timestamp,
                     sql_text
                FROM dba_common_audit_trail)
  LOOP
    DBMS_OUTPUT.put_line (   'Audit Type: '
                          || rec.audit_type);
    DBMS_OUTPUT.put_line (   'Who:        '
                          || rec.db_user);
    DBMS_OUTPUT.put_line (   'What:       '
                          || rec.object_schema
                          || '.'
                          || rec.object_name);
    DBMS_OUTPUT.put_line (   'When:       '
                          || rec.extended_timestamp);
    DBMS_OUTPUT.put_line (   'How:        '
                          || rec.sql_text);
    DBMS_OUTPUT.put_line
      ('--------------- End of Record ---------------');
  END LOOP;
END;
/

NOAUDIT SELECT ON scott.emp;
-- nothing in audit trail
DELETE FROM SYS.aud$;
COMMIT ;
SELECT COUNT (*)
  FROM SYS.dba_audit_trail;

CONN system/manager

DROP USER aud_mgr CASCADE;
CREATE USER aud_mgr IDENTIFIED BY saodfu90
  DEFAULT TABLESPACE users
  TEMPORARY TABLESPACE temp;
GRANT CREATE SESSION, CREATE PROCEDURE TO aud_mgr;
GRANT CREATE ANY TRIGGER TO aud_mgr;
GRANT ADMINISTER DATABASE TRIGGER TO aud_mgr;

CONN sys/oracle as sysdba
GRANT SELECT ON v_$session TO aud_mgr;
GRANT SELECT ON v_$process TO aud_mgr;
GRANT SELECT ON dba_common_audit_trail TO aud_mgr;

CONN aud_mgr/saodfu90
CREATE OR REPLACE TRIGGER set_default_client_info
  AFTER LOGON ON DATABASE
DECLARE
  l_module  v$session.module%TYPE;
BEGIN
  SELECT UPPER (module)
    INTO l_module
    FROM v$process a, v$session b
   WHERE a.addr = b.paddr
     AND b.audsid = USERENV ('sessionid');
  DBMS_SESSION.set_identifier
                      (   SYS_CONTEXT
                                     ('userenv',
                                      'ip_address')
                       || ':'
                       || l_module);
END;
/
AUDIT SELECT ON scott.emp BY ACCESS;

CONN system/manager
SELECT ename, sal
  FROM scott.emp
 WHERE sal < (SELECT sal
                FROM scott.emp
               WHERE ename = 'SCOTT')
   AND deptno = (SELECT deptno
                   FROM scott.emp
                  WHERE ename = 'SCOTT');


-- grant privs to demo to select on scott.emp
CONN scott/tiger

-- set client identifier, this shows up in audit trail
--exec dbms_session.set_identifier('Application APP_A');
--exec DBMS_APPLICATION_INFO.SET_CLIENT_INFO (sys_context('userenv','ip_address'));

-- execute three very different queries
SELECT SUM (sal)
  FROM scott.emp;
SELECT ename
  FROM scott.emp
 WHERE 1 = 2;

CONN aud_mgr/am

CREATE OR REPLACE PROCEDURE show_aud
AS
BEGIN
  FOR rec IN (SELECT   db_user,
                       client_id,
                       object_schema,
                       object_name,
                       extended_timestamp,
                       sql_text,
                       statementid
                  FROM dba_common_audit_trail
              GROUP BY db_user,
                       statementid,
                       sql_text,
                       object_schema,
                       object_name,
                       client_id,
                       extended_timestamp
              ORDER BY extended_timestamp ASC)
  LOOP
    DBMS_OUTPUT.put_line ('Who:  ' || rec.db_user);
    DBMS_OUTPUT.put_line (   'What: '
                          || rec.object_schema
                          || '.'
                          || rec.object_name);
    DBMS_OUTPUT.put_line (   'Where: '
                          || rec.client_id);
    DBMS_OUTPUT.put_line
                 (   'When:  '
                  || TO_CHAR
                          (rec.extended_timestamp,
                           'Mon-DD HH24:MI'));
    DBMS_OUTPUT.put_line ('How:   '
                          || rec.sql_text);
    DBMS_OUTPUT.put_line
      ('-------------- End of Record --------------');
  END LOOP;
END;
/

EXEC show_aud


--------- timing ---------------
SET timing on

BEGIN
  FOR rec IN 1 .. 100000
  LOOP
    FOR irec IN (SELECT ename
                   FROM scott.emp)
    LOOP
      NULL;
    END LOOP;
  END LOOP;
END;
/

-- connect as a user without access


DELETE FROM SYS.aud$;
NOAUDIT SELECT ON scott.emp;
COMMIT ;
REVOKE SELECT ON scott.emp FROM demo;


EDIT audit.log
EXIT
