SPOOL fga.log
CONN system/manager
-- standard auditing call
-- "audit select on scott.emp by access;"

-- FGA call

BEGIN
  DBMS_FGA.add_policy
              (object_schema      => 'SCOTT',
               object_name        => 'EMP',
               policy_name        => 'Example',
               audit_condition    => 'ENAME != USER');
END;
/


-- nothing in FGA audit trail
DELETE FROM SYS.dba_fga_audit_trail;
COMMIT ;
SELECT COUNT (*)
  FROM SYS.dba_fga_audit_trail;

-- grant privs to demo to select on scott.emp
CONN scott/tiger
ANALYZE TABLE emp COMPUTE STATISTICS;

SELECT sal, comm FROM emp
 WHERE ename = 'SCOTT';
SELECT sal, comm FROM emp
 WHERE deptno = 20;

CREATE OR REPLACE FUNCTION is_off_hours
  RETURN BINARY_INTEGER
AS
  l_return_val  BINARY_INTEGER;
  l_day_number  VARCHAR2 (1)
                        := TO_CHAR (SYSDATE, 'D');
  l_hour        VARCHAR2 (2)
                     := TO_CHAR (SYSDATE, 'HH24');
BEGIN
  IF (   l_day_number IN ('1', '7')
      OR l_hour < 8
      OR l_hour > 18)
  THEN
    l_return_val := 1;
  ELSE
    l_return_val := 0;
  END IF;

  RETURN l_return_val;
END;
/

BEGIN
  IF (is_off_hours = TRUE)
  THEN
    DBMS_OUTPUT.put_line ('is off hours');
  ELSE
    DBMS_OUTPUT.put_line
              ('is NOT off hours, we are working');
  END IF;
END;
/

BEGIN
  DBMS_FGA.drop_policy (object_schema    => 'SCOTT',
                        object_name      => 'EMP',
                        policy_name      => 'Example');
  DBMS_FGA.add_policy
    (object_schema      => 'SCOTT',
     object_name        => 'EMP',
     policy_name        => 'Example',
     audit_condition    => 'SEC_MGR.IS_OFF_HOURS = 1');
END;
/

GRANT EXECUTE ON is_off_hours TO scott;

SELECT ename
  FROM scott.emp
 WHERE 1 = 2;

------------------ column sensititvy ---------------

BEGIN
  DBMS_FGA.drop_policy (object_schema    => 'SCOTT',
                        object_name      => 'EMP',
                        policy_name      => 'Example');
END;
BEGIN
  DBMS_FGA.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'EMP',
              policy_name        => 'Example',
              audit_condition    => 'ENAME != USER',
              audit_column       => 'SAL');
END;
/

-- Aggregate value on sensitive column causes audit
SELECT SUM (sal) FROM scott.emp;
-- This will not audit since ename = user
SELECT sal FROM emp
 WHERE ename = 'SCOTT';
-- Direct query on column will audit 
SELECT empno, job, sal
  FROM emp
 WHERE deptno = 10;
-- No Audit since SAL column is not queried
SELECT empno, job
  FROM emp
 WHERE deptno = 10;
-- No audit since no records are returned 
SELECT ename FROM emp
 WHERE ename = 'KNOX';



------------------ events ---------------
CREATE OR REPLACE PROCEDURE fga_notify (
  object_schema  VARCHAR2,
  object_name    VARCHAR2,
  policy_name    VARCHAR2)
AS
  l_message    VARCHAR2 (32767);
  l_mailhost   VARCHAR2 (30)
                  := 'rgmamersmtp.oraclecorp.com';
  l_mail_conn  UTL_SMTP.connection;
  l_from       VARCHAR2 (30)
                        := 'FGA_ADMIN@oracle.com';
  l_to         VARCHAR2 (30)
                       := 'david.knox@oracle.com';
BEGIN
  l_message :=
       'User '
    || USER
    || ' successfully accessed '
    || object_schema
    || '.'
    || object_name
    || ' at '
    || TO_CHAR (SYSDATE, 'Month DD HH24:MI:SS')
    || ' with this statement:  "'
    || SYS_CONTEXT ('userenv', 'current_sql')
    || '"';
  l_mail_conn :=
         UTL_SMTP.open_connection (l_mailhost, 25);
  UTL_SMTP.helo (l_mail_conn, l_mailhost);
  UTL_SMTP.mail (l_mail_conn, l_from);
  UTL_SMTP.rcpt (l_mail_conn, l_to);
  UTL_SMTP.DATA (l_mail_conn,
                    UTL_TCP.crlf
                 || 'Subject: FGA Alert'
                 || UTL_TCP.crlf
                 || 'To: '
                 || l_to
                 || UTL_TCP.crlf
                 || l_message);
  UTL_SMTP.quit (l_mail_conn);
EXCEPTION
  WHEN OTHERS
  THEN
    UTL_SMTP.quit (l_mail_conn);
    raise_application_error
        (-20000,
            'Failed due to the following error: '
         || SQLERRM);
END;
/


BEGIN
  DBMS_FGA.drop_policy (object_schema    => 'SCOTT',
                        object_name      => 'EMP',
                        policy_name      => 'Example');
  DBMS_FGA.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'EMP',
              policy_name        => 'Example',
              audit_condition    => 'ENAME != USER',
              audit_column       => 'SAL',
              handler_schema     => 'SEC_MGR',
              handler_module     => 'FGA_NOTIFY');
END;
/


-- Oracle 9i test 
conn system/manager
BEGIN
  DBMS_FGA.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'EMP',
              policy_name        => 'Example',
              audit_condition    => 'ENAME != USER',
              audit_column       => 'SAL',
              handler_schema     => 'NO_SCHEMA',
              handler_module     => 'UNDEFINED');
END;
/

CONN scott/tiger
SELECT COUNT (ename) FROM emp;
SELECT ename, sal FROM emp;

------------------ updates, what the data looked like, and undo -------
CONN scott/tiger
CREATE OR REPLACE VIEW emp_view
AS
  SELECT ename,
         DECODE (ename, USER, sal, NULL) sal,
         deptno
    FROM emp;
SELECT ename, deptno, sal
  FROM emp_view;
GRANT SELECT ON emp_view TO blake;

CONN sec_mgr/oracle10g

BEGIN
  DBMS_FGA.drop_policy
                    (object_schema    => 'SCOTT',
                     object_name      => 'EMP_VIEW',
                     policy_name      => 'ViewExample');
END;
/

BEGIN
  DBMS_FGA.add_policy
             (object_schema      => 'SCOTT',
              object_name        => 'EMP_VIEW',
              policy_name        => 'ViewExample',
              audit_condition    => 'ENAME != USER',
              audit_column       => 'SAL');
END;
/

-- look at dept 20 salaries
SELECT ename, sal
  FROM emp_view
 WHERE deptno = 30;

connect 
update scott.emp
set deptno=20 where deptno = 30;

SELECT ename, sal
  FROM emp
 WHERE deptno = 30;
commit;
