CONN scott/tiger

CREATE OR REPLACE TRIGGER update_emp_sal_trig
  BEFORE UPDATE OF sal
  ON emp_copy
  FOR EACH ROW
DECLARE
BEGIN
  audit_emp (p_username       => USER,
             p_action         => 'UPDATE',
             p_empno          => :OLD.empno,
             p_column_name    => 'SAL',
             p_old_value      => TO_CHAR (:OLD.sal),
             p_new_value      => TO_CHAR (:NEW.sal));
END;
/
GRANT UPDATE(sal) ON emp_copy TO blake;

CONN blake/blake

UPDATE scott.emp_copy
   SET sal = sal * 1.1
 WHERE deptno = 20;
COMMIT ;

CONN scott/tiger
EXEC show_aud_emp;

SELECT DISTINCT call_stack
           FROM aud_emp;

TRUNCATE TABLE aud_emp;

CONN blake/blake

SELECT SUM (sal)
  FROM scott.emp_copy
 WHERE deptno = 20;
UPDATE scott.emp_copy
   SET sal = sal * 1.1
 WHERE deptno = 20;
SELECT SUM (sal)
  FROM scott.emp_copy
 WHERE deptno = 20;
ROLLBACK ;
SELECT SUM (sal)
  FROM scott.emp_copy
 WHERE deptno = 20;

CONN scott/tiger
EXEC show_aud_emp;

CREATE OR REPLACE PROCEDURE audit_emp (
  p_username     IN  VARCHAR2,
  p_action       IN  VARCHAR2,
  p_empno        IN  NUMBER,
  p_column_name  IN  VARCHAR2,
  p_old_value    IN  VARCHAR2,
  p_new_value    IN  VARCHAR2)
AS
  PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
  INSERT INTO aud_emp
              (username,
               action,
               empno,
               column_name,
               call_stack,
               client_id,
               old_value,
               new_value,
               action_date)
       VALUES (p_username,
               p_action,
               p_empno,
               p_column_name,
               DBMS_UTILITY.format_call_stack,
               SYS_CONTEXT ('userenv',
                            'client_identifier'),
               p_old_value,
               p_new_value,
               SYSDATE);
  COMMIT;
END;
/

CONN scott/tiger
DELETE FROM aud_emp;
COMMIT ;

-- should see audit even though BLAKE rollsback
CONN blake/blake
UPDATE scott.emp_copy
   SET sal = sal * 1.1
 WHERE deptno = 20;
ROLLBACK ;

CONN scott/tiger
SELECT COUNT (*)
  FROM aud_emp;
EXEC show_aud_emp

