CONN scott/tiger
drop table emp_copy;
CREATE TABLE emp_copy AS SELECT * FROM emp;

DROP TABLE aud_emp;
CREATE TABLE aud_emp (
  username    VARCHAR2(30),
  action      VARCHAR2(6),
  empno       NUMBER(4),
  column_name VARCHAR2(255),
  call_stack  VARCHAR2(4000),
  client_id   VARCHAR2(255),
  old_value   VARCHAR2(10),
  new_value   VARCHAR2(10),
  action_date DATE DEFAULT SYSDATE
)
/

CREATE OR REPLACE PROCEDURE audit_emp (
  p_username     IN  VARCHAR2,
  p_action       IN  VARCHAR2,
  p_empno        IN  NUMBER,
  p_column_name  IN  VARCHAR2,
  p_old_value    IN  VARCHAR2,
  p_new_value    IN  VARCHAR2)
AS
BEGIN
-- check data format and length
-- not shown here
  INSERT INTO aud_emp
              (username,
               action,
               empno,
               column_name,
               call_stack,
               client_id,
               old_value,
               new_value,
               action_date)
       VALUES (p_username,
               p_action,
               p_empno,
               p_column_name,
               DBMS_UTILITY.format_call_stack,
               SYS_CONTEXT ('userenv',
                            'client_identifier'),
               p_old_value,
               p_new_value,
               SYSDATE);
END;
/

-- create procedure to display audit trail records
CREATE OR REPLACE PROCEDURE show_aud_emp
AS
BEGIN
  FOR rec IN (SELECT   *
                  FROM aud_emp
              ORDER BY action_date DESC)
  LOOP
    DBMS_OUTPUT.put_line (   'User:       '
                          || rec.username);
    DBMS_OUTPUT.put_line (   'Client ID:  '
                          || rec.client_id);
    DBMS_OUTPUT.put_line (   'Action:     '
                          || rec.action);
    DBMS_OUTPUT.put_line (   'Empno:      '
                          || rec.empno);
    DBMS_OUTPUT.put_line (   'Column:     '
                          || rec.column_name);
    DBMS_OUTPUT.put_line (   'Old Value:  '
                          || rec.old_value);
    DBMS_OUTPUT.put_line (   'New Value:  '
                          || rec.new_value);
    DBMS_OUTPUT.put_line (   'Date:       '
                          || TO_CHAR
                              (rec.action_date,
                               'Mon-DD-YY HH24:MI'));
    DBMS_OUTPUT.put_line
                 ('------------------------------');
  END LOOP;
END;
/

CREATE OR REPLACE PROCEDURE update_sal (
  p_empno   IN  NUMBER,
  p_salary  IN  NUMBER)
AS
  l_old_sal  VARCHAR2 (10);
BEGIN
  SELECT     sal
        INTO l_old_sal
        FROM emp_copy
       WHERE empno = p_empno
  FOR UPDATE;
  UPDATE emp_copy
     SET sal = p_salary
   WHERE empno = p_empno;
  audit_emp
    (p_username       => USER,
     p_action         => 'UPDATE',
     p_empno          => p_empno,
     p_column_name    => 'SAL',
     p_old_value      => l_old_sal,
     p_new_value      => p_salary);
END;
/

GRANT EXECUTE ON update_sal TO blake;
GRANT SELECT ON emp_copy TO blake;

CONN sec_mgr/oracle10g
CREATE OR REPLACE TRIGGER set_ip_in_id
  AFTER LOGON ON DATABASE
BEGIN
  DBMS_SESSION.set_identifier
                       (SYS_CONTEXT ('userenv',
                                     'ip_address'));
END;
/
CONN blake/blake
SELECT empno, sal
  FROM scott.emp_copy
 WHERE ename = 'BLAKE';
EXEC scott.update_sal(p_empno=>7698, p_salary=>3000);
COMMIT ;
SELECT empno, sal
  FROM scott.emp_copy
 WHERE ename = 'BLAKE';

CONN scott/tiger
EXEC show_aud_emp
COL call_stack format a50
COL username format a10
SELECT username, call_stack
  FROM aud_emp;


-- drop table aud_emp;
-- drop table emp_copy;
