CREATE ROLE tigr_admin IDENTIFIED GLOBALLY;
CREATE  ROLE tigr_user IDENTIFIED GLOBALLY;

-- database cannot control global role authorizations
GRANT tigr_user TO scott;

CREATE OR REPLACE FUNCTION user_is_admin
  RETURN VARCHAR2 AUTHID CURRENT_USER
AS
  l_return_val  VARCHAR2 (4) := 'NO';
                            -- does NOT have role
BEGIN
  FOR rec IN (SELECT ROLE
                FROM session_roles
               WHERE ROLE = 'TIGR_ADMIN')
  LOOP
    l_return_val := 'YES';
  END LOOP;

  RETURN l_return_val;
END;
/

CREATE OR REPLACE VIEW dr_view
AS
  SELECT SYSDATE "Date"
    FROM DUAL
   WHERE user_is_admin = 'YES';


CREATE USER tigr_app_global IDENTIFIED GLOBALLY AS '';
GRANT EXECUTE ON user_is_admin TO tigr_app_global;
GRANT SELECT ON dr_view TO tigr_app_global;

CONN king/oracle9i
SELECT *
  FROM session_roles;
SELECT sec_mgr.user_is_admin
  FROM DUAL;
SELECT *
  FROM sec_mgr.dr_view;

conn sec_mgr/oracle10g
DROP CONTEXT admin_ctx;
CREATE CONTEXT admin_ctx USING sec_mgr.admin_ctx;

CREATE OR REPLACE PROCEDURE admin_ctx
AUTHID CURRENT_USER
AS
BEGIN
  FOR rec IN (SELECT ROLE
                FROM session_roles
               WHERE ROLE = 'TIGR_ADMIN')
  LOOP
    DBMS_SESSION.set_context
                      (namespace    => 'admin_ctx',
                       ATTRIBUTE    => 'TIGR_ADMIN',
                       VALUE        => 'YES');
  END LOOP;
END;
/

GRANT EXECUTE ON admin_ctx TO tigr_app_global;

CREATE OR REPLACE FUNCTION user_is_admin
  RETURN VARCHAR2 AUTHID CURRENT_USER
AS
BEGIN
  RETURN NVL (SYS_CONTEXT ('admin_ctx',
                           'TIGR_ADMIN'),
              'NO');
END;
/

CONN king/oracle9i
EXEC sec_mgr.admin_ctx;
SELECT *
  FROM sec_mgr.dr_view;
