conn sec_mgr/oracle10g

DROP TABLE asset$ CASCADE CONSTRAINTS;

CREATE TABLE asset$
(
  ITEMID          VARCHAR2(30 BYTE),
  ITEM            VARCHAR2(255 BYTE),
  READINESS_CODE  VARCHAR2(16 BYTE)
);
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'AC1234516', 'USS John F. Kennedy  (CV 67) ', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'AA1234517', 'USS Wasp (LHD 1)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'D1234516', 'USS Spruance (DD 963) ', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'D1234518', 'USS Roosevelt (DDG 80)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'F2234503', 'USS Underwood (FFG 36)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'F2234504', 'USS Taylor (FFG 50)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'C2234506', 'USS Vicksburg(CG 69) ', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'ATD2234508', 'USS Trenton (LPD 14)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'S3234501', 'USS Boise (SSN 764)', 'c-1'); 
INSERT INTO asset$ ( ITEMID, ITEM, READINESS_CODE ) VALUES ( 
'S3234502', 'USS Toledo(SSN 769)', 'c-1'); 
COMMIT;


-- need privileges to create a context
DROP CONTEXT global_threat_ctx;
-- create a global context indicating current
-- threat level
CREATE CONTEXT global_threat_ctx
  USING sec_mgr.global_ctx_mgr ACCESSED GLOBALLY;

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_threat_level  IN  VARCHAR2)
AS
BEGIN
  DBMS_SESSION.set_context
                  (NAMESPACE    => 'global_threat_ctx',
                   ATTRIBUTE    => 'threat_level',
                   VALUE        => p_threat_level);
END;
/

EXEC global_ctx_mgr ('normal');
-- create a view based on value of context
-- show all records when value is normal
-- mask column when value is elevated
-- and show no records when value is critical
CREATE OR REPLACE VIEW assetd
AS
  SELECT itemid,
         item,
         DECODE
               (SYS_CONTEXT ('global_threat_ctx',
                             'threat_level'),
                'elevated', NULL,
                readiness_code) status
    FROM asset$
   WHERE SYS_CONTEXT ('global_threat_ctx',
                      'threat_level') !=
                                        'critical';
select item, status from assetd;
EXEC global_ctx_mgr ('elevated');
/
EXEC global_ctx_mgr ('critical');
/


------------- simplified records -----------
-- create a view based on value of context
-- show all records when value is normal
-- show no records when value is critical
CREATE OR REPLACE VIEW assets
AS
  SELECT itemid,
         item,
         readiness_code
    FROM asset$
   WHERE SYS_CONTEXT ('global_threat_ctx',
                      'threat_level') !=
                                        'elevated';
EXEC global_ctx_mgr ('normal');
select item, status from assets;
EXEC global_ctx_mgr ('elevated');
/

COL threat_level format a25
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.clear_identifier;
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level')
  FROM DUAL;

----------- global for all sessions of same user ------
CREATE USER army IDENTIFIED BY army;
GRANT CREATE SESSION TO army;
GRANT SELECT ON assets TO army;
CREATE USER navy IDENTIFIED BY navy;
GRANT CREATE SESSION TO navy;
GRANT SELECT ON assets TO navy;
CREATE PUBLIC SYNONYM assets FOR assets;

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_threat_level  IN  VARCHAR2,
  p_user    IN  VARCHAR2 DEFAULT USER)
AS
BEGIN
  DBMS_SESSION.set_context
                  (namespace    => 'global_threat_ctx',
                   ATTRIBUTE    => 'threat_level',
                   VALUE        => p_threat_level,
                   username     => p_user);
END;
/

EXEC global_ctx_mgr('Day HH24:MI', 'SEC_MGR');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN sec_mgr
SELECT get_timestamp "Date/Time"
  FROM DUAL;

CONN scott/tiger
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;

CONN sec_mgr/sec_mgr
EXEC global_ctx_mgr(p_user=>'SCOTT');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN scott/tiger
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;

------------- overloading ------------

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_threat_level     IN  VARCHAR2,
  p_user       IN  VARCHAR2 DEFAULT NULL,
  p_client_id  IN  VARCHAR2)
AS
BEGIN
  DBMS_SESSION.set_context
                  (namespace    => 'global_threat_ctx',
                   ATTRIBUTE    => 'threat_level',
                   VALUE        => p_threat_level,
                   username     => p_user,
                   client_id    => p_client_id);
END;
/

SHOW errors

BEGIN
  global_ctx_mgr
              (p_threat_level       => 'Day HH24:MI:SS',
               p_client_id    => 'Application Alpha');
  global_ctx_mgr
                (p_threat_level       => 'MM DD',
                 p_client_id    => 'Application Beta');
END;
/

COL "DATE/TIME" format a50
-- Client ID is not set, value is NULL
SELECT get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Alpha');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN scott/tiger
EXEC dbms_session.set_identifier('Application Alpha');
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/

--------- username and client Id set --------------

BEGIN
  global_ctx_mgr
          (p_threat_level       => 'SEC_MGR User, ID of "A"',
           p_user         => 'SEC_MGR',
           p_client_id    => 'A');
  global_ctx_mgr
           (p_threat_level       => 'SEC_MGR User, ID of "B"',
            p_user         => 'SEC_MGR',
            p_client_id    => 'B');
END;
/

COL Value format a30
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') "Value"
  FROM DUAL;
EXEC dbms_session.set_identifier('B');
/

BEGIN
  global_ctx_mgr
          (p_threat_level       => 'SEC_MGR User, ID of "A"',
           p_user         => 'SEC_MGR',
           p_client_id    => 'A');
  global_ctx_mgr
             (p_threat_level       => 'SCOTT User, ID of "A"',
              p_user         => 'SCOTT',
              p_client_id    => 'A');
END;
/
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') "Value"
  FROM DUAL;
CONN scott/tiger
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') "Value"
  FROM DUAL;
EXEC dbms_session.set_identifier('B');
/


