CONN sec_mgr/oracle10g

-- need privileges to create a context
DROP CONTEXT global_threat_ctx;
-- create a global context indicating current
-- threat level
CREATE CONTEXT global_threat_ctx
  USING sec_mgr.global_ctx_mgr ACCESSED GLOBALLY;


CREATE OR REPLACE PACKAGE global_ctx_mgr
AS
  PROCEDURE set_level (p_threat_level IN VARCHAR2);
  PROCEDURE CLEAR;
END;
/

CREATE OR REPLACE PACKAGE BODY global_ctx_mgr
AS
  PROCEDURE set_level (p_threat_level IN VARCHAR2)
  AS
  BEGIN
    DBMS_SESSION.set_context
               (namespace    => 'global_threat_ctx',
                ATTRIBUTE    => 'threat_level',
                VALUE        => p_threat_level);
  END;
-----------------------------
  PROCEDURE CLEAR
  AS
  BEGIN
    DBMS_SESSION.clear_all_context
                             ('global_threat_ctx');
  END;
END;
/

EXEC global_ctx_mgr.set_level ('normal');
COL threat_level format a25
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;

DROP TABLE asset$ CASCADE CONSTRAINTS;

CREATE TABLE asset$
(
  itemid          VARCHAR2(30),
  item            VARCHAR2(255),
  status          VARCHAR2(16)
);
INSERT INTO asset$
            (itemid, item, status)
     VALUES ('AC1234516',
             'USS John F. Kennedy  (CV 67) ',
             'c-1');
COMMIT ;
------------- simplified records -----------
-- create a view based on value of context
-- show all records when value is normal
-- show no records when value is elevated or undefined
CREATE OR REPLACE VIEW assets
AS
  SELECT itemid, item, status
    FROM asset$
   WHERE 1 = decode(SYS_CONTEXT ('global_threat_ctx',
                      'threat_level'), 'normal',1,'elevated',0,0);
GRANT SELECT ON assets TO scott;

-- test View
EXEC global_ctx_mgr.set_level ('normal');
COL item format a30
COL status format a6
SELECT item, status FROM assets;
-- value exists for SCOTT too
CONN scott/tiger
CREATE SYNONYM assets FOR sec_mgr.assets;
SELECT item, status FROM assets;
CONN sec_mgr/oracle10g
-- change in value changes result set
EXEC global_ctx_mgr.set_level ('elevated');
-- will not get any records since value is "elevated"
SELECT count(*) FROM assets;
CONN scott/tiger
SELECT count(*) FROM assets;

----------- global for all sessions of same user ------
CONN sec_mgr/oracle10g
CREATE OR REPLACE PACKAGE global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_user          IN  VARCHAR2);

  PROCEDURE CLEAR;
END;
/
CREATE OR REPLACE PACKAGE BODY global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_user          IN  VARCHAR2)
  AS
  BEGIN
    DBMS_SESSION.set_context
               (namespace    => 'global_threat_ctx',
                ATTRIBUTE    => 'threat_level',
                VALUE        => p_threat_level,
                username     => p_user);
  END;

  PROCEDURE CLEAR
  AS
  BEGIN
    DBMS_SESSION.clear_all_context
                             ('global_threat_ctx');
  END;
END;
/

EXEC global_ctx_mgr.set_level ('normal', 'SEC_MGR');
SELECT item, status FROM assets;
-- SCOTT no longer has a value for this
CONN scott/tiger
SELECT item, status FROM assets;

CONN sec_mgr/oracle10g
-- set the value for SCOTT
-- this "erases" the value that was stored for SEC_MGR
EXEC global_ctx_mgr.set_level ('normal', 'SCOTT');
SELECT item, status FROM assets;
-- SCOTT can see the value
CONN scott/tiger
SELECT item, status FROM assets;

--------------- changing Client ID ------------
CONN sec_mgr/oracle10g
CREATE OR REPLACE PACKAGE global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_client_id     IN  VARCHAR2);
  PROCEDURE CLEAR;
END;
/
CREATE OR REPLACE PACKAGE BODY global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_client_id     IN  VARCHAR2)
  AS
  BEGIN
    DBMS_SESSION.set_context
               (namespace    => 'global_threat_ctx',
                ATTRIBUTE    => 'threat_level',
                VALUE        => p_threat_level,
                client_id    => p_client_id);
  END;
  PROCEDURE CLEAR
  AS
  BEGIN
    DBMS_SESSION.clear_all_context
                             ('global_threat_ctx');
  END;
END;
/
-- clear the namespace
EXEC global_ctx_mgr.clear;
-- set global context for ID --
BEGIN
  global_ctx_mgr.set_level ('App A Value',
                            'Application Alpha');
  global_ctx_mgr.set_level ('App B Value',
                            'Application Beta');
END;
/
-- set Client ID
EXEC dbms_session.set_identifier('Application Alpha');
COL threat_level format a25
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;

CONN scott/tiger
-- Client ID is not set, so no value
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
-- set ID allows access to value
EXEC dbms_session.set_identifier('Application Alpha');
/
EXEC dbms_session.set_identifier('Application Beta');
/

------------- overloading ------------
-- different values for same schema with same ID

CREATE OR REPLACE PACKAGE global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_user          IN  VARCHAR2 DEFAULT NULL,
    p_client_id     IN  VARCHAR2);

  PROCEDURE CLEAR;
END;
/
CREATE OR REPLACE PACKAGE BODY global_ctx_mgr
AS
  PROCEDURE set_level (
    p_threat_level  IN  VARCHAR2,
    p_user          IN  VARCHAR2 DEFAULT NULL,
    p_client_id     IN  VARCHAR2)
  AS
  BEGIN
    DBMS_SESSION.set_context
               (namespace    => 'global_threat_ctx',
                ATTRIBUTE    => 'threat_level',
                VALUE        => p_threat_level,
                username     => p_user,
                client_id    => p_client_id);
  END;

  PROCEDURE CLEAR
  AS
  BEGIN
    DBMS_SESSION.clear_all_context
                             ('global_threat_ctx');
  END;
END;
/
-- clear the namespace
EXEC global_ctx_mgr.clear;
-- set values for the SEC_MGR user
BEGIN
  global_ctx_mgr.set_level
    (p_threat_level    => 'ClientID Alpha:SEC_MGR value',
     p_user            => 'SEC_MGR',
     p_client_id       => 'Application Alpha');
  global_ctx_mgr.set_level
    (p_threat_level    => 'ClientID Beta:SEC_MGR value',
     p_user            => 'SEC_MGR',
     p_client_id       => 'Application Beta');
END;
/
-- show values
EXEC dbms_session.set_identifier('Application Alpha');
COL threat_level format a30
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/
CONN scott/tiger
-- SCOTT cannot see values
EXEC dbms_session.set_identifier('Application Alpha');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/
--------- username and client Id set --------------
CONN sec_mgr/oracle10g

BEGIN
  global_ctx_mgr.set_level
    (p_threat_level    => 'ClientID Alpha:SEC_MGR value',
     p_user            => 'SEC_MGR',
     p_client_id       => 'Application Alpha');
  -- this call erases the one just set
  global_ctx_mgr.set_level
    (p_threat_level    => 'ClientID Alpha:SCOTT value',
     p_user            => 'SCOTT',
     p_client_id       => 'Application Alpha');
END;
/

EXEC dbms_session.set_identifier('Application Alpha');
COL threat_level format a30
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/
CONN scott/tiger
EXEC dbms_session.set_identifier('Application Alpha');
SELECT SYS_CONTEXT ('global_threat_ctx',
                    'threat_level') threat_level
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/
