SPOOL global_app_ctx.log

-- need privileges to create a context
CONN system/manager
DROP CONTEXT global_app_ctx;
CREATE CONTEXT global_app_ctx
  USING sec_mgr.global_ctx_mgr ACCESSED GLOBALLY;

CONN SEC_MGR/SEC_MGR

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_format  IN  VARCHAR2
      DEFAULT 'Day Month DD HH24:MI:SS')
AS
BEGIN
  DBMS_SESSION.set_context
                  (NAMESPACE    => 'global_app_ctx',
                   ATTRIBUTE    => 'date_format',
                   VALUE        => p_format);
END;
/

SHOW errors

EXEC global_ctx_mgr
COL date_format format a25
SELECT SYS_CONTEXT ('global_app_ctx',
                    'date_format') DATE_FORMAT
  FROM DUAL;
EXEC dbms_session.clear_identifier;
SELECT SYS_CONTEXT ('global_app_ctx',
                    'date_format')
  FROM DUAL;

CREATE OR REPLACE FUNCTION get_timestamp
  RETURN VARCHAR2
AS
BEGIN
  RETURN TO_CHAR (SYSDATE,
                  SYS_CONTEXT ('global_app_ctx',
                               'date_format'));
END;
/

SHOW errors

COL "DATE/TIME" format a50
SELECT get_timestamp "Date/Time"
  FROM DUAL;
GRANT EXECUTE ON get_timestamp TO PUBLIC;
EXEC global_ctx_mgr('MM-DD HH:MI');
CONN scott/tiger
COL "DATE/TIME" format a50
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;

----------- global for all sessions of same user ------

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_format  IN  VARCHAR2
      DEFAULT 'Day Month DD HH24:MI:SS',
  p_user    IN  VARCHAR2 DEFAULT USER)
AS
BEGIN
  DBMS_SESSION.set_context
                  (namespace    => 'global_app_ctx',
                   ATTRIBUTE    => 'date_format',
                   VALUE        => p_format,
                   username     => p_user);
END;
/

COL "DATE/TIME" format a50
EXEC global_ctx_mgr('Day HH24:MI', 'SEC_MGR');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN sec_mgr
SELECT get_timestamp "Date/Time"
  FROM DUAL;

CONN scott/tiger
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;

CONN sec_mgr/sec_mgr
EXEC global_ctx_mgr(p_user=>'SCOTT');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN scott/tiger
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;

------------- overloading ------------

CREATE OR REPLACE PROCEDURE global_ctx_mgr (
  p_format     IN  VARCHAR2,
  p_user       IN  VARCHAR2 DEFAULT NULL,
  p_client_id  IN  VARCHAR2)
AS
BEGIN
  DBMS_SESSION.set_context
                  (namespace    => 'global_app_ctx',
                   ATTRIBUTE    => 'date_format',
                   VALUE        => p_format,
                   username     => p_user,
                   client_id    => p_client_id);
END;
/

SHOW errors

BEGIN
  global_ctx_mgr
              (p_format       => 'Day HH24:MI:SS',
               p_client_id    => 'Application Alpha');
  global_ctx_mgr
                (p_format       => 'MM DD',
                 p_client_id    => 'Application Beta');
END;
/

COL "DATE/TIME" format a50
-- Client ID is not set, value is NULL
SELECT get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Alpha');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
SELECT get_timestamp "Date/Time"
  FROM DUAL;
CONN scott/tiger
EXEC dbms_session.set_identifier('Application Alpha');
SELECT sec_mgr.get_timestamp "Date/Time"
  FROM DUAL;
EXEC dbms_session.set_identifier('Application Beta');
/

--------- username and client Id set --------------

BEGIN
  global_ctx_mgr
          (p_format       => 'SEC_MGR User, ID of "A"',
           p_user         => 'SEC_MGR',
           p_client_id    => 'A');
  global_ctx_mgr
           (p_format       => 'SEC_MGR User, ID of "B"',
            p_user         => 'SEC_MGR',
            p_client_id    => 'B');
END;
/

COL Value format a30
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_app_ctx',
                    'date_format') "Value"
  FROM DUAL;
EXEC dbms_session.set_identifier('B');
/

BEGIN
  global_ctx_mgr
          (p_format       => 'SEC_MGR User, ID of "A"',
           p_user         => 'SEC_MGR',
           p_client_id    => 'A');
  global_ctx_mgr
             (p_format       => 'SCOTT User, ID of "A"',
              p_user         => 'SCOTT',
              p_client_id    => 'A');
END;
/
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_app_ctx',
                    'date_format') "Value"
  FROM DUAL;
CONN scott/tiger
EXEC dbms_session.set_identifier('A');
SELECT SYS_CONTEXT ('global_app_ctx',
                    'date_format') "Value"
  FROM DUAL;
EXEC dbms_session.set_identifier('B');
/


